# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: d6_TempHum.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10\x64\x36_TempHum.proto\"\x13\n\x11\x64\x36\x63\x31\x30\x30\x30_th_get_rq\":\n\x11\x64\x36\x63\x31\x30\x30\x30_th_get_ra\x12\x13\n\x0btemperature\x18\x01 \x01(\x11\x12\x10\n\x08humidity\x18\x02 \x01(\rb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'd6_TempHum_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _D6C1000_TH_GET_RQ._serialized_start=20
  _D6C1000_TH_GET_RQ._serialized_end=39
  _D6C1000_TH_GET_RA._serialized_start=41
  _D6C1000_TH_GET_RA._serialized_end=99
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: d1B_Lcd.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rd1B_Lcd.proto\"\x17\n\x15\x64\x31\x62\x63\x31\x30\x30\x31_lcd_clear_rq\"\x17\n\x15\x64\x31\x62\x63\x31\x30\x30\x31_lcd_clear_ra\"/\n\x1e\x64\x31\x62\x63\x31\x30\x30\x32_lcd_back_light_set_rq\x12\r\n\x05value\x18\x01 \x01(\r\" \n\x1e\x64\x31\x62\x63\x31\x30\x30\x32_lcd_back_light_set_ra\"5\n$d1bc1003_lcd_background_color_set_rq\x12\r\n\x05\x63olor\x18\x01 \x01(\r\"&\n$d1bc1003_lcd_background_color_set_ra\";\n\x1b\x64\x31\x62\x63\x31\x30\x30\x34_lcd_bar_display_rq\x12\x0e\n\x06status\x18\x01 \x01(\r\x12\x0c\n\x04text\x18\x02 \x01(\x0c\"\x1d\n\x1b\x64\x31\x62\x63\x31\x30\x30\x34_lcd_bar_display_ra\"Z\n\x1b\x64\x31\x62\x63\x31\x30\x30\x35_lcd_static_text_rq\x12\x0c\n\x04text\x18\x01 \x01(\x0c\x12\r\n\x05\x63olor\x18\x02 \x01(\r\x12\x0e\n\x06post_x\x18\x03 \x01(\r\x12\x0e\n\x06post_y\x18\x04 \x01(\r\"\x1d\n\x1b\x64\x31\x62\x63\x31\x30\x30\x35_lcd_static_text_ra\"X\n\x19\x64\x31\x62\x63\x31\x30\x30\x36_lcd_roll_text_rq\x12\x0c\n\x04text\x18\x01 \x01(\x0c\x12\r\n\x05\x63olor\x18\x02 \x01(\r\x12\x0e\n\x06post_x\x18\x03 \x01(\r\x12\x0e\n\x06post_y\x18\x04 \x01(\r\"\x1b\n\x19\x64\x31\x62\x63\x31\x30\x30\x36_lcd_roll_text_ra\"S\n\x14\x64\x31\x62\x63\x31\x30\x30\x37_lcd_icon_rq\x12\x0c\n\x04type\x18\x01 \x01(\r\x12\r\n\x05index\x18\x02 \x01(\r\x12\x0e\n\x06post_x\x18\x03 \x01(\r\x12\x0e\n\x06post_y\x18\x04 \x01(\r\"\x16\n\x14\x64\x31\x62\x63\x31\x30\x30\x37_lcd_icon_ra\"J\n\x19\x64\x31\x62\x63\x31\x30\x30\x38_lcd_inner_pic_rq\x12\r\n\x05index\x18\x01 \x01(\r\x12\x0e\n\x06post_x\x18\x02 \x01(\r\x12\x0e\n\x06post_y\x18\x03 \x01(\r\"\x1b\n\x19\x64\x31\x62\x63\x31\x30\x30\x38_lcd_inner_pic_ra\"9\n\x1b\x64\x31\x62\x63\x31\x30\x30\x39_lcd_pic_display_rq\x12\x0c\n\x04type\x18\x01 \x01(\r\x12\x0c\n\x04name\x18\x02 \x01(\t\"\x1d\n\x1b\x64\x31\x62\x63\x31\x30\x30\x39_lcd_pic_display_ra\":\n\x1e\x64\x31\x62\x63\x31\x30\x31\x30_lcd_sensor_display_rq\x12\x0b\n\x03opt\x18\x01 \x01(\r\x12\x0b\n\x03\x64\x65v\x18\x02 \x01(\r\" \n\x1e\x64\x31\x62\x63\x31\x30\x31\x30_lcd_sensor_display_rab\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'd1B_Lcd_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _D1BC1001_LCD_CLEAR_RQ._serialized_start=17
  _D1BC1001_LCD_CLEAR_RQ._serialized_end=40
  _D1BC1001_LCD_CLEAR_RA._serialized_start=42
  _D1BC1001_LCD_CLEAR_RA._serialized_end=65
  _D1BC1002_LCD_BACK_LIGHT_SET_RQ._serialized_start=67
  _D1BC1002_LCD_BACK_LIGHT_SET_RQ._serialized_end=114
  _D1BC1002_LCD_BACK_LIGHT_SET_RA._serialized_start=116
  _D1BC1002_LCD_BACK_LIGHT_SET_RA._serialized_end=148
  _D1BC1003_LCD_BACKGROUND_COLOR_SET_RQ._serialized_start=150
  _D1BC1003_LCD_BACKGROUND_COLOR_SET_RQ._serialized_end=203
  _D1BC1003_LCD_BACKGROUND_COLOR_SET_RA._serialized_start=205
  _D1BC1003_LCD_BACKGROUND_COLOR_SET_RA._serialized_end=243
  _D1BC1004_LCD_BAR_DISPLAY_RQ._serialized_start=245
  _D1BC1004_LCD_BAR_DISPLAY_RQ._serialized_end=304
  _D1BC1004_LCD_BAR_DISPLAY_RA._serialized_start=306
  _D1BC1004_LCD_BAR_DISPLAY_RA._serialized_end=335
  _D1BC1005_LCD_STATIC_TEXT_RQ._serialized_start=337
  _D1BC1005_LCD_STATIC_TEXT_RQ._serialized_end=427
  _D1BC1005_LCD_STATIC_TEXT_RA._serialized_start=429
  _D1BC1005_LCD_STATIC_TEXT_RA._serialized_end=458
  _D1BC1006_LCD_ROLL_TEXT_RQ._serialized_start=460
  _D1BC1006_LCD_ROLL_TEXT_RQ._serialized_end=548
  _D1BC1006_LCD_ROLL_TEXT_RA._serialized_start=550
  _D1BC1006_LCD_ROLL_TEXT_RA._serialized_end=577
  _D1BC1007_LCD_ICON_RQ._serialized_start=579
  _D1BC1007_LCD_ICON_RQ._serialized_end=662
  _D1BC1007_LCD_ICON_RA._serialized_start=664
  _D1BC1007_LCD_ICON_RA._serialized_end=686
  _D1BC1008_LCD_INNER_PIC_RQ._serialized_start=688
  _D1BC1008_LCD_INNER_PIC_RQ._serialized_end=762
  _D1BC1008_LCD_INNER_PIC_RA._serialized_start=764
  _D1BC1008_LCD_INNER_PIC_RA._serialized_end=791
  _D1BC1009_LCD_PIC_DISPLAY_RQ._serialized_start=793
  _D1BC1009_LCD_PIC_DISPLAY_RQ._serialized_end=850
  _D1BC1009_LCD_PIC_DISPLAY_RA._serialized_start=852
  _D1BC1009_LCD_PIC_DISPLAY_RA._serialized_end=881
  _D1BC1010_LCD_SENSOR_DISPLAY_RQ._serialized_start=883
  _D1BC1010_LCD_SENSOR_DISPLAY_RQ._serialized_end=941
  _D1BC1010_LCD_SENSOR_DISPLAY_RA._serialized_start=943
  _D1BC1010_LCD_SENSOR_DISPLAY_RA._serialized_end=975
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: d9_Ultrasonic.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13\x64\x39_Ultrasonic.proto\"\x18\n\x16\x64\x39\x63\x31\x30\x30\x30_ult_dis_get_rq\"*\n\x16\x64\x39\x63\x31\x30\x30\x30_ult_dis_get_ra\x12\x10\n\x08\x64istance\x18\x01 \x01(\r\"f\n\x18\x64\x39\x63\x31\x30\x30\x31_ult_light_set_rq\x12\t\n\x01R\x18\x01 \x01(\r\x12\t\n\x01G\x18\x02 \x01(\r\x12\t\n\x01\x42\x18\x03 \x01(\r\x12\x0c\n\x04mode\x18\x04 \x01(\r\x12\r\n\x05speed\x18\x05 \x01(\r\x12\x0c\n\x04time\x18\x06 \x01(\r\"\x1a\n\x18\x64\x39\x63\x31\x30\x30\x31_ult_light_set_rab\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'd9_Ultrasonic_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _D9C1000_ULT_DIS_GET_RQ._serialized_start=23
  _D9C1000_ULT_DIS_GET_RQ._serialized_end=47
  _D9C1000_ULT_DIS_GET_RA._serialized_start=49
  _D9C1000_ULT_DIS_GET_RA._serialized_end=91
  _D9C1001_ULT_LIGHT_SET_RQ._serialized_start=93
  _D9C1001_ULT_LIGHT_SET_RQ._serialized_end=195
  _D9C1001_ULT_LIGHT_SET_RA._serialized_start=197
  _D9C1001_ULT_LIGHT_SET_RA._serialized_end=223
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dE_VoiceRecognition.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19\x64\x45_VoiceRecognition.proto\"C\n\x17\x64\x65\x63\x31\x30\x32\x32_vr_file_list_rq\x12\x13\n\x0b\x66ile_format\x18\x01 \x01(\r\x12\x13\n\x0bsuffix_type\x18\x02 \x01(\r\"\x99\x01\n\x17\x64\x65\x63\x31\x30\x32\x32_vr_file_list_ra\x12\x32\n\x04list\x18\x01 \x03(\x0b\x32$.dec1022_vr_file_list_ra.file_info_t\x1aJ\n\x0b\x66ile_info_t\x12\x11\n\tfile_name\x18\x01 \x01(\t\x12\x15\n\rfile_duration\x18\x02 \x01(\r\x12\x11\n\tfile_date\x18\x03 \x01(\r\"p\n\x1f\x64\x65\x63\x31\x30\x32\x36_vr_effect_play_start_rq\x12\x11\n\tfile_type\x18\x01 \x01(\t\x12\x11\n\tfile_name\x18\x02 \x01(\t\x12\x13\n\x0b\x66ile_format\x18\x03 \x01(\r\x12\x12\n\nsession_id\x18\x04 \x01(\r\"!\n\x1f\x64\x65\x63\x31\x30\x32\x36_vr_effect_play_start_ra\"]\n\x1f\x64\x65\x63\x31\x30\x32\x37_vr_record_play_start_rq\x12\x11\n\tfile_name\x18\x01 \x01(\t\x12\x13\n\x0b\x66ile_format\x18\x02 \x01(\r\x12\x12\n\nsession_id\x18\x03 \x01(\r\"!\n\x1f\x64\x65\x63\x31\x30\x32\x37_vr_record_play_start_ra\"\x1e\n\x1c\x64\x65\x63\x31\x30\x32\x38_vr_file_play_stop_rq\"\x1e\n\x1c\x64\x65\x63\x31\x30\x32\x38_vr_file_play_stop_ra\"L\n\x16\x64\x65\x63\x31\x30\x33\x30_vr_tts_play_rq\x12\x10\n\x08language\x18\x01 \x01(\t\x12\x0c\n\x04text\x18\x02 \x01(\t\x12\x12\n\nsession_id\x18\x03 \x01(\r\"\x18\n\x16\x64\x65\x63\x31\x30\x33\x30_vr_tts_play_ra\")\n\x18\x64\x65\x63\x31\x30\x33\x35_vr_set_volume_rq\x12\r\n\x05value\x18\x01 \x01(\r\"\x1a\n\x18\x64\x65\x63\x31\x30\x33\x35_vr_set_volume_ra\"\x1a\n\x18\x64\x65\x63\x31\x30\x33\x36_vr_get_volume_rq\")\n\x18\x64\x65\x63\x31\x30\x33\x36_vr_get_volume_ra\x12\r\n\x05value\x18\x01 \x01(\r\"B\n\x16\x64\x65\x63\x31\x30\x33\x39_vr_file_num_rq\x12\x13\n\x0b\x66ile_format\x18\x01 \x01(\r\x12\x13\n\x0bsuffix_type\x18\x02 \x01(\r\"*\n\x16\x64\x65\x63\x31\x30\x33\x39_vr_file_num_ra\x12\x10\n\x08\x66ile_num\x18\x01 \x01(\r\"\x18\n\x16\x64\x65\x63\x31\x30\x34\x31_vr_tts_stop_rq\"\x18\n\x16\x64\x65\x63\x31\x30\x34\x31_vr_tts_stop_ra\"\x18\n\x16\x64\x65\x63\x31\x30\x34\x32_vr_dir_init_rq\"\x18\n\x16\x64\x65\x63\x31\x30\x34\x32_vr_dir_init_ra\"\x17\n\x15\x64\x65\x63\x31\x30\x34\x33_vr_dir_get_rq\"$\n\x15\x64\x65\x63\x31\x30\x34\x33_vr_dir_get_ra\x12\x0b\n\x03\x64ir\x18\x01 \x01(\r\"+\n\x17\x64\x65\x63\x31\x30\x33\x32_vr_asr_start_rq\x12\x10\n\x08language\x18\x01 \x01(\t\"\x19\n\x17\x64\x65\x63\x31\x30\x33\x32_vr_asr_start_ra\"\x18\n\x16\x64\x65\x63\x31\x30\x33\x33_vr_asr_stop_rq\"\x18\n\x16\x64\x65\x63\x31\x30\x33\x33_vr_asr_stop_ra\"\x1c\n\x1a\x64\x65\x63\x31\x30\x34\x34_vr_asr_get_text_rq\"*\n\x1a\x64\x65\x63\x31\x30\x34\x34_vr_asr_get_text_ra\x12\x0c\n\x04text\x18\x01 \x01(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dE_VoiceRecognition_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DEC1022_VR_FILE_LIST_RQ._serialized_start=29
  _DEC1022_VR_FILE_LIST_RQ._serialized_end=96
  _DEC1022_VR_FILE_LIST_RA._serialized_start=99
  _DEC1022_VR_FILE_LIST_RA._serialized_end=252
  _DEC1022_VR_FILE_LIST_RA_FILE_INFO_T._serialized_start=178
  _DEC1022_VR_FILE_LIST_RA_FILE_INFO_T._serialized_end=252
  _DEC1026_VR_EFFECT_PLAY_START_RQ._serialized_start=254
  _DEC1026_VR_EFFECT_PLAY_START_RQ._serialized_end=366
  _DEC1026_VR_EFFECT_PLAY_START_RA._serialized_start=368
  _DEC1026_VR_EFFECT_PLAY_START_RA._serialized_end=401
  _DEC1027_VR_RECORD_PLAY_START_RQ._serialized_start=403
  _DEC1027_VR_RECORD_PLAY_START_RQ._serialized_end=496
  _DEC1027_VR_RECORD_PLAY_START_RA._serialized_start=498
  _DEC1027_VR_RECORD_PLAY_START_RA._serialized_end=531
  _DEC1028_VR_FILE_PLAY_STOP_RQ._serialized_start=533
  _DEC1028_VR_FILE_PLAY_STOP_RQ._serialized_end=563
  _DEC1028_VR_FILE_PLAY_STOP_RA._serialized_start=565
  _DEC1028_VR_FILE_PLAY_STOP_RA._serialized_end=595
  _DEC1030_VR_TTS_PLAY_RQ._serialized_start=597
  _DEC1030_VR_TTS_PLAY_RQ._serialized_end=673
  _DEC1030_VR_TTS_PLAY_RA._serialized_start=675
  _DEC1030_VR_TTS_PLAY_RA._serialized_end=699
  _DEC1035_VR_SET_VOLUME_RQ._serialized_start=701
  _DEC1035_VR_SET_VOLUME_RQ._serialized_end=742
  _DEC1035_VR_SET_VOLUME_RA._serialized_start=744
  _DEC1035_VR_SET_VOLUME_RA._serialized_end=770
  _DEC1036_VR_GET_VOLUME_RQ._serialized_start=772
  _DEC1036_VR_GET_VOLUME_RQ._serialized_end=798
  _DEC1036_VR_GET_VOLUME_RA._serialized_start=800
  _DEC1036_VR_GET_VOLUME_RA._serialized_end=841
  _DEC1039_VR_FILE_NUM_RQ._serialized_start=843
  _DEC1039_VR_FILE_NUM_RQ._serialized_end=909
  _DEC1039_VR_FILE_NUM_RA._serialized_start=911
  _DEC1039_VR_FILE_NUM_RA._serialized_end=953
  _DEC1041_VR_TTS_STOP_RQ._serialized_start=955
  _DEC1041_VR_TTS_STOP_RQ._serialized_end=979
  _DEC1041_VR_TTS_STOP_RA._serialized_start=981
  _DEC1041_VR_TTS_STOP_RA._serialized_end=1005
  _DEC1042_VR_DIR_INIT_RQ._serialized_start=1007
  _DEC1042_VR_DIR_INIT_RQ._serialized_end=1031
  _DEC1042_VR_DIR_INIT_RA._serialized_start=1033
  _DEC1042_VR_DIR_INIT_RA._serialized_end=1057
  _DEC1043_VR_DIR_GET_RQ._serialized_start=1059
  _DEC1043_VR_DIR_GET_RQ._serialized_end=1082
  _DEC1043_VR_DIR_GET_RA._serialized_start=1084
  _DEC1043_VR_DIR_GET_RA._serialized_end=1120
  _DEC1032_VR_ASR_START_RQ._serialized_start=1122
  _DEC1032_VR_ASR_START_RQ._serialized_end=1165
  _DEC1032_VR_ASR_START_RA._serialized_start=1167
  _DEC1032_VR_ASR_START_RA._serialized_end=1192
  _DEC1033_VR_ASR_STOP_RQ._serialized_start=1194
  _DEC1033_VR_ASR_STOP_RQ._serialized_end=1218
  _DEC1033_VR_ASR_STOP_RA._serialized_start=1220
  _DEC1033_VR_ASR_STOP_RA._serialized_end=1244
  _DEC1044_VR_ASR_GET_TEXT_RQ._serialized_start=1246
  _DEC1044_VR_ASR_GET_TEXT_RQ._serialized_end=1274
  _DEC1044_VR_ASR_GET_TEXT_RA._serialized_start=1276
  _DEC1044_VR_ASR_GET_TEXT_RA._serialized_end=1318
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: d20_VisionModule.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16\x64\x32\x30_VisionModule.proto\"%\n#d20c1003_visionmodule_mid_offset_rq\"5\n#d20c1003_visionmodule_mid_offset_ra\x12\x0e\n\x06offset\x18\x01 \x01(\x02\"2\n!d20c1005_visionmodule_identify_rq\x12\r\n\x05model\x18\x01 \x01(\x11\"\xe7\x01\n!d20c1005_visionmodule_identify_ra\x12\r\n\x05model\x18\x01 \x01(\x11\x12>\n\x04info\x18\x02 \x03(\x0b\x32\x30.d20c1005_visionmodule_identify_ra.identify_info\x1as\n\ridentify_info\x12\x0b\n\x03tid\x18\x01 \x01(\x11\x12\r\n\x05tname\x18\x02 \x01(\t\x12\x0b\n\x03\x65id\x18\x03 \x01(\x11\x12\r\n\x05\x65name\x18\x04 \x01(\t\x12\t\n\x01x\x18\x05 \x01(\r\x12\t\n\x01y\x18\x06 \x01(\r\x12\t\n\x01w\x18\x07 \x01(\r\x12\t\n\x01h\x18\x08 \x01(\r\"6\n%d20c1008_visionmodule_model_switch_rq\x12\r\n\x05model\x18\x01 \x01(\x11\"7\n%d20c1008_visionmodule_model_switch_ra\x12\x0e\n\x06result\x18\x01 \x01(\x11\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'd20_VisionModule_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _D20C1003_VISIONMODULE_MID_OFFSET_RQ._serialized_start=26
  _D20C1003_VISIONMODULE_MID_OFFSET_RQ._serialized_end=63
  _D20C1003_VISIONMODULE_MID_OFFSET_RA._serialized_start=65
  _D20C1003_VISIONMODULE_MID_OFFSET_RA._serialized_end=118
  _D20C1005_VISIONMODULE_IDENTIFY_RQ._serialized_start=120
  _D20C1005_VISIONMODULE_IDENTIFY_RQ._serialized_end=170
  _D20C1005_VISIONMODULE_IDENTIFY_RA._serialized_start=173
  _D20C1005_VISIONMODULE_IDENTIFY_RA._serialized_end=404
  _D20C1005_VISIONMODULE_IDENTIFY_RA_IDENTIFY_INFO._serialized_start=289
  _D20C1005_VISIONMODULE_IDENTIFY_RA_IDENTIFY_INFO._serialized_end=404
  _D20C1008_VISIONMODULE_MODEL_SWITCH_RQ._serialized_start=406
  _D20C1008_VISIONMODULE_MODEL_SWITCH_RQ._serialized_end=460
  _D20C1008_VISIONMODULE_MODEL_SWITCH_RA._serialized_start=462
  _D20C1008_VISIONMODULE_MODEL_SWITCH_RA._serialized_end=517
# @@protoc_insertion_point(module_scope)



from ukitai.link import *
from ukitai.common import *
from ukitai.apis import *


def set_state(*, id: int, enabled: bool, link: uKitAiLink):
    """打开/关闭传感器功能

    参数值说明:
        id: 传感器 id

        enabled: True=开打传感器功能，False=关闭传感器功能

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    return CommonApi.comp_state(compType=CompType.COLOR, id=id, enabled=enabled, link=link)


def read_red_value(*, id: int, link: uKitAiLink):
    """读取颜色值红色分量

    参数值说明:
        id: 传感器 id

        link: 设备连接

    返回值说明:
        (int): 颜色值红色分量，范围[0, 255]，读取失败时返回None
    """
    return CommonApi.read_value(id=id, data_type=DataType.COLOR_RED, link=link)


def read_blue_value(*, id: int, link: uKitAiLink):
    """读取颜色值蓝色分量

    参数值说明:
        id: 传感器 id

        link: 设备连接

    返回值说明:
        (int): 颜色值蓝色分量，范围[0, 255]，读取失败时返回None
    """
    return CommonApi.read_value(id=id, data_type=DataType.COLOR_BLUE, link=link)


def read_green_value(*, id: int, link: uKitAiLink):
    """读取颜色值绿色分量

    参数值说明:
        id: 传感器 id

        link: 设备连接

    返回值说明:
        (int): 颜色值绿色分量，范围[0, 255]，读取失败时返回None
    """
    return CommonApi.read_value(id=id, data_type=DataType.COLOR_GREEN, link=link)



from ukitai.common import *
from ukitai.link import *
from ukitai.protos import *


def comp_state(*, compType: int, id: int, enabled: bool, link: uKitAiLink):
    message = None
    pbReq = None
    pbRsp = None
    if enabled:
        cmd = 9
    else:
        cmd = 10
    return link.sendRequest(compType, id, cmd, message, pbReq, pbRsp)


def read_value(*, id: int = 0, data_type: DataType, link: uKitAiLink):
    message = None
    pbReq = None
    valueKey = None

    def default_handle(x):
        return x

    handle_func = default_handle
    if data_type == DataType.INFRARED_DISTANCE:
        pbRsp = d5_Infrared_pb2.d5c1001_ir_dis_get_ra()
        dev = CompType.INFRARED
        cmd = 1001
        valueKey = 'distance'
    elif data_type == DataType.ULTRASONIC_DISTANCE:
        pbRsp = d9_Ultrasonic_pb2.d9c1000_ult_dis_get_ra()
        dev = CompType.ULTRASOUND
        cmd = 1000
        valueKey = 'distance'
    elif data_type == DataType.SOUND_STRONG:
        pbRsp = d7_Sound_pb2.d7c1000_snd_adc_value_get_ra()
        dev = CompType.SOUND
        cmd = 1000
        valueKey = 'adc_value'
    elif data_type == DataType.BRIGHTNESS:
        pbRsp = d11_Luxmeter_pb2.d11c1000_lgt_value_get_ra()
        dev = CompType.ENV_LIGHT
        cmd = 1000
        valueKey = 'value'
    elif data_type == DataType.HUMIDITY:
        pbRsp = d6_TempHum_pb2.d6c1000_th_get_ra()
        dev = CompType.HUMITURE
        cmd = 1000
        valueKey = 'humidity'
    elif data_type == DataType.TEMPERATURE_C:
        pbRsp = d6_TempHum_pb2.d6c1000_th_get_ra()
        dev = CompType.HUMITURE
        cmd = 1000
        valueKey = 'temperature'
    elif data_type == DataType.TEMPERATURE_F:
        pbRsp = d6_TempHum_pb2.d6c1000_th_get_ra()
        dev = CompType.HUMITURE
        cmd = 1000
        valueKey = 'temperature'

        def fahrenheit_handle(x):
            return 32 + int(x * 1.8)

        handle_func = fahrenheit_handle
    elif data_type == DataType.COLOR_RED:
        pbRsp = dD_Color_pb2.ddc1000_clr_rgb_get_ra()
        dev = CompType.COLOR
        cmd = 1000
        valueKey = 'rgbc'

        def color_red_handle(x):
            return (x >> 24) & 0xFF

        handle_func = color_red_handle
    elif data_type == DataType.COLOR_GREEN:
        pbRsp = dD_Color_pb2.ddc1000_clr_rgb_get_ra()
        dev = CompType.COLOR
        cmd = 1000
        valueKey = 'rgbc'

        def color_green_handle(x):
            return (x >> 16) & 0xFF

        handle_func = color_green_handle
    elif data_type == DataType.COLOR_BLUE:
        pbRsp = dD_Color_pb2.ddc1000_clr_rgb_get_ra()
        dev = CompType.COLOR
        cmd = 1000
        valueKey = 'rgbc'

        def color_blue_handle(x):
            return (x >> 8) & 0xFF

        handle_func = color_blue_handle
    elif data_type == DataType.SERVO_ANGLE:
        message = {'pwr': False}
        pbReq = d3_Servo_pb2.d3c1001_srv_angle_get_rq()
        pbRsp = d3_Servo_pb2.d3c1001_srv_angle_get_ra()
        dev = CompType.SERVO
        cmd = 1001
        valueKey = 'cur_angle'

        def angle_handle(x):
            return (x / 10) - 120

        handle_func = angle_handle
    elif data_type == DataType.BATTERY_LEVEL:
        pbRsp = dF_Battery_pb2.dfc1000_bat_pwr_pct_ra()
        dev = CompType.BATTERY
        cmd = 1000
        valueKey = 'pwr_percent'
    elif data_type == DataType.BATTERY_STATUS:
        pbRsp = dF_Battery_pb2.dfc1001_bat_status_get_ra()
        dev = CompType.BATTERY
        cmd = 1001
        valueKey = 'bat_status'
    elif data_type == DataType.CHARGE_STATUS:
        pbRsp = dF_Battery_pb2.dfc1001_bat_status_get_ra()
        dev = CompType.BATTERY
        cmd = 1001
        valueKey = 'adapter_status'
    elif data_type == DataType.TOUCH_STATUS:
        pbRsp = dB_TouchSwitch_pb2.dbc1001_tch_type_get_ra()
        dev = CompType.TOUCH
        cmd = 1001
        valueKey = 'type'
    else:
        # 不支持的类型
        return None
    ack, response = link.sendRequest(dev, id, cmd, message, pbReq, pbRsp)
    if ack != 0 or not response:
        # 请求失败
        return None
    value = getattr(response, valueKey)
    return handle_func(value)



from ukitai.link import *
from ukitai.common import *
from ukitai.apis import *


def read_brightness(*, id: int, link: uKitAiLink):
    """读取亮度值

    参数值说明:
        id: 传感器 id

        link: 设备连接

    返回值说明:
        (int): 亮度值，单位lux，读取失败时返回None
    """
    return CommonApi.read_value(id=id, data_type=DataType.BRIGHTNESS, link=link)



import _thread
from time import sleep

import serial
import serial.tools.list_ports

from ukitai.link.uKitAiLink import uKitAiLink


class uKitAiSerialLink(uKitAiLink):
    _port = None
    _serial = None

    def _background_received_thread(self):
        while self._serial.isOpen():
            try:
                remainLen = self._serial.inWaiting()
                if remainLen > 0:
                    data = self._serial.read(remainLen)
                    self._onReceivedData(data)
                sleep(0.01)
            except Exception:
                pass

    def __init__(self, port: str):
        super().__init__()
        self._port = port

    def isOpen(self) -> bool:
        """判断是否已经和目标设备成功建立连接

        返回值说明:
            (bool): 是否已经成功连接设备
        """
        return self._serial is not None and self._serial.isOpen()

    def open(self, timeout: float = 0.5, **kwargs) -> bool:
        """与目标设备建立连接

        参数值说明:
            timeout (float): 超时时间，单位秒

        返回值说明:
            (bool): 是否已经成功连接设备
        """
        try:
            if self._serial and self._serial.isOpen():
                return False
            self._serial = serial.Serial(port=self._port, baudrate=115200, timeout=timeout, **kwargs)
            if not self._serial.isOpen():
                self._serial.open()
            if self._serial.isOpen():
                _thread.start_new_thread(self._background_received_thread, ())
            return self._serial.isOpen()
        except Exception:
            self.close()
            return False

    def close(self) -> bool:
        """中断已建立的连接并释放资源

        返回值说明:
            (bool): 是否已经断开设备连接
        """
        try:
            if self._serial.isOpen():
                self._serial.close()
            return not self._serial.isOpen()
        except Exception:
            return False

    def _sendData(self, data: bytes) -> int:
        try:
            if self._serial.isOpen():
                return self._serial.write(data)
            else:
                return -1
        except Exception:
            return -1


def getDevices():
    """列出当前通过串口识别到的所有uKitAi设备信息

        返回值说明:
            (list): 设备信息列表
        """
    deviceInfos = []
    try:
        port_list = list(serial.tools.list_ports.comports())
        if len(port_list) > 0:
            for dev in list(port_list):
                if dev.vid == 0x0403 and dev.pid == 0x6001:
                    deviceInfos.append({'manufacturer': dev.manufacturer, 'product': dev.product, 'device': dev.device})
    except Exception:
        pass
    return deviceInfos


def listDevices():
    """列出当前通过串口识别到的所有uKitAi设备信息

    返回值说明:
        (None): 通过控制台显示结果
    """
    index = 0
    deviceInfos = getDevices()
    for deviceInfo in deviceInfos:
        index = index + 1
        print("Device %d: %s %s\n\t%s" % (index, deviceInfo['manufacturer'], deviceInfo['product'], deviceInfo['device']))


def create(port: str = None) -> uKitAiSerialLink:
    """创建一个uKitAi串口连接对象

    参数值说明:
        port (str): 串口设备端口

        在Linux系统上面格式一般为：'/dev/ttyUSBXXXX'

        在MacOS系统上面格式一般为：'/dev/tty.usbserial-XXXX'

        在Windows系统上面格式一般为：'COMXXXX'

    返回值说明:
        (bool): 返回创建好的uKitAi串口连接对象
    """
    return uKitAiSerialLink(port)



from enum import Enum

from ukitai.common import *
from ukitai.link import *
from ukitai.protos import *


class MotorDirection(Enum):
    """运动方向

    CLOCKWISE: 顺时针

    ANTICLOCKWISE: 逆时针
    """
    CLOCKWISE = 1
    ANTICLOCKWISE = -1


def turn_motor(*, id: int, direction: MotorDirection, speed: int, link: uKitAiLink):
    """控制电机按照指定的方向和转速转动

    参数值说明:
        id: 电机 id
        direction (Motor.MotorDirection): 指定转动方向
        speed (int): 转速, 单位: 转/分, 范围:[0, 140]
        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据

    示例:
        # 控制 id 为 1 的电机以50 转/分的速度逆时针

        Motor.turn_motor(id=1, direction=Motor.MotorDirection.ANTICLOCKWISE, speed=50, link=link)

        # 控制 id 为 2 的电机以100 转/分的速度顺时针转动

        Motor.turn_motor(id=2, direction=Motor.MotorDirection.CLOCKWISE, speed=100, link=link)
    """
    speed = int(Common.num_normal(speed, 140, 0)) * direction.value
    message = {'speed': speed, 'time': 0xFFFFFFFF}
    pbReq = d4_Motor_pb2.d4c1000_mtr_speed_set_rq()
    pbRsp = d4_Motor_pb2.d4c1000_mtr_speed_set_ra()
    return link.sendRequest(CompType.MOTOR, id, 1000, message, pbReq, pbRsp)


def turn_motor_pwm(*, id: int, direction: MotorDirection, pwm: int, link: uKitAiLink):
    """控制电机按照指定的方向和PWM转速转动

    参数值说明:
        id: 电机 id
        direction (Motor.MotorDirection): 指定转动方向
        pwm (int): pwm转速, 范围:[0, 1000]
        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    pwm = int(Common.num_normal(pwm, 1000, 0)) * direction.value
    message = {'pwm': pwm}
    pbReq = d4_Motor_pb2.d4c1004_mtr_pwm_set_rq()
    pbRsp = d4_Motor_pb2.d4c1004_mtr_pwm_set_ra()
    return link.sendRequest(CompType.MOTOR, id, 1004, message, pbReq, pbRsp)


def stop_motor(*, id: int, link: uKitAiLink):
    """停止指定的电机

    参数值说明:
        id: 电机 id
        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据

    示例:
        # 停止 id 为 5 的电机的转动

        Motor.stop_motor(id=5, link=link)

        # 停止 id 为 2 的电机的转动

        Motor.stop_motor(id=2, link=link)
    """
    message = None
    pbReq = None
    pbRsp = None
    return link.sendRequest(CompType.MOTOR, id, 1001, message, pbReq, pbRsp)



from abc import ABCMeta, abstractmethod


class LinkInterface(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def isOpen(self) -> bool:
        pass

    @abstractmethod
    def open(self, timeout=0.5, **kwargs) -> bool:
        pass

    @abstractmethod
    def close(self) -> bool:
        pass

    @abstractmethod
    def _sendData(self, data: bytes) -> int:
        pass

    @abstractmethod
    def _onReceivedData(self, data: bytes):
        pass



from ukitai.common import *
from ukitai.link import *
from ukitai.protos import *
from ukitai.apis import *


def set_state(*, id: int, enabled: bool, link: uKitAiLink):
    """打开/关闭传感器功能

    参数值说明:
        id: 传感器 id

        enabled: True=开打传感器功能，False=关闭传感器功能

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    return CommonApi.comp_state(compType=CompType.ULTRASOUND, id=id, enabled=enabled, link=link)


def read_distance(*, id: int, link: uKitAiLink):
    """读取障碍物距离

    参数值说明:
        id: 传感器 id

        link: 设备连接

    返回值说明:
        (int): 距离，单位cm，范围[0, 400]，读取失败时返回None
    """
    return CommonApi.read_value(id=id, data_type=DataType.ULTRASONIC_DISTANCE, link=link)


def show_color_rgb(*, id: int, color: Color, time: int, link: uKitAiLink):
    """设置灯光颜色

    参数值说明:
        id: 传感器 id

        color (Color): 颜色值

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    if time < 0 or time >= 6553500:
        time = 0xFFFFFFFF
        speed = 0xFF
    else:
        time = int(Common.num_normal(time, 6553499, 0))
        speed = 0
    message = {'R': color.red(), 'G': color.green(), 'B': color.blue(), 'mode': 1, 'time': time, 'speed': speed}
    pbReq = d9_Ultrasonic_pb2.d9c1001_ult_light_set_rq()
    pbRsp = d9_Ultrasonic_pb2.d9c1001_ult_light_set_ra()
    return link.sendRequest(CompType.ULTRASOUND, id, 1001, message, pbReq, pbRsp)


def turn_off_light(*, id: int, link: uKitAiLink):
    """关闭灯光

    参数值说明:
        id: 传感器 id

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'R': 0, 'G': 0, 'B': 0, 'mode': 0, 'time': 0, 'speed': 0}
    pbReq = d9_Ultrasonic_pb2.d9c1001_ult_light_set_rq()
    pbRsp = d9_Ultrasonic_pb2.d9c1001_ult_light_set_ra()
    return link.sendRequest(CompType.ULTRASOUND, id, 1001, message, pbReq, pbRsp)



import math
from ukitai.common import *


class Color(object):
    """ 定义颜色类型 """
    color = 0

    def __init__(self, color=0):
        self.color = color

    def red(self) -> int:
        """ 获取颜色值红色分量

        返回值说明:
            (int): 范围[0, 255]
        """
        return (self.color & 0x00FF0000) >> 16

    def green(self) -> int:
        """ 获取颜色值绿色分量

        返回值说明:
            (int): 范围[0, 255]
        """
        return (self.color & 0x0000FF00) >> 8

    def blue(self) -> int:
        """ 获取颜色值蓝色分量

        返回值说明:
            (int): 范围[0, 255]
        """
        return self.color & 0x000000FF


def create_color_rgb(red: int, green: int, blue: int) -> Color:
    """根据红, 绿, 蓝分量值创建颜色

    参数值说明:
        red (int): 红通分量

        green (int): 绿通分量

        blue (int): 蓝通分量

    返回值说明:
        (Color): 返回 Color 对象
    """
    v_red = Common.num_normal(red, 255, 0)
    v_green = Common.num_normal(green, 255, 0)
    v_blue = Common.num_normal(blue, 255, 0)
    v = v_red << 16 | v_green << 8 | v_blue
    return Color(v)


def __create_color_hsv(hue: int, saturation: int, value: int):
    """[summary]

    参数值说明:
        hue (int): 色调值, 范围: [0, 360]

        saturation (int): 饱和度, 范围: [0, 100]

        value (int): 明度, 范围: [0, 100]
    """
    h = int(hue) % 360
    s = Common.num_normal(saturation, 100, 0) * 0.01
    v = Common.num_normal(value, 100, 0) * 0.01

    i = math.floor(h / 60)
    f = (h / 60) - i
    p = v * (1 - s)
    q = v * (1 - (s * f))
    t = v * (1 - (s * (1 - f)))

    if i == 0:
        rr = (v, t, p)
    elif i == 1:
        rr = (q, v, p)
    elif i == 2:
        rr = (p, v, t)
    elif i == 3:
        rr = (p, q, v)
    elif i == 4:
        rr = (t, p, v)
    else:
        rr = (v, p, q)

    result = Color.create_color_rgb(math.floor(rr[0] * 255),
                                    math.floor(rr[1] * 255),
                                    math.floor(rr[2] * 255))
    # print("===== create_color_hsv:", result.color)
    return result


def create_color_hsv(hue_percent: int, saturation: int, value: int) -> Color:
    """根据色调, 饱和度, 亮度值创建颜色

    参数值说明:
        hue_percent (int): 色调值百分比, 范围: [0, 100]

        saturation (int): 饱和度, 范围: [0, 100]

        value (int): 明度, 范围: [0, 100]

    返回值说明:
        (Color): 返回 Color 对象
    """
    p = Common.num_normal(hue_percent, 100, 0)
    return __create_color_hsv(int(p * 360 * 0.01), saturation, value)


RED = Color(0x00FF0000)
ORANGE = Color(0x00FF8000)
YELLOW = Color(0x00FFF000)
GREEN = Color(0x0000FF00)
CYAN = Color(0x0000FFFF)
BLUE = Color(0x000000FF)
PURPLE = Color(0x00FF00FF)
BLACK = Color(0x00000000)
WHITE = Color(0x00FFFFFF)
GREY = Color(0x00BEBEBE)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: d5_Infrared.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x64\x35_Infrared.proto\"\x17\n\x15\x64\x35\x63\x31\x30\x30\x31_ir_dis_get_rq\")\n\x15\x64\x35\x63\x31\x30\x30\x31_ir_dis_get_ra\x12\x10\n\x08\x64istance\x18\x01 \x01(\rb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'd5_Infrared_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _D5C1001_IR_DIS_GET_RQ._serialized_start=21
  _D5C1001_IR_DIS_GET_RQ._serialized_end=44
  _D5C1001_IR_DIS_GET_RA._serialized_start=46
  _D5C1001_IR_DIS_GET_RA._serialized_end=87
# @@protoc_insertion_point(module_scope)



import time
from enum import Enum

from ukitai.common import *
from ukitai.link import *
from ukitai.protos import *
from ukitai.apis import *


class VisionModel(Enum):
    """模型定义

    TRAFFIC: 交通模型

    FACE: 人脸识别

    FACE_COUNT: 人脸数量

    GENDER: 性别

    MASK: 口罩

    EMOTION: 情绪

    GESTURE: 手势

    MODEL_TOYS: 模型小车小人

    TRAFFIC_LIGHT_MODEL: 模型红绿灯

    GARBAGE: 垃圾

    HANDWRITTEN_DIGIT: 数字

    HANDWRITTEN_LETTER: 字母

    COLOR: 指定识别物颜色识别

    CUSTOM_COLOR: 自定义颜色识别

    CUSTOM_AI_MODULE: 在线识别

    LOCAL_CUSTOM_MODEL: 本地自定义模型
    """
    TRAFFIC = 0x00
    FACE = 0x01
    FACE_COUNT = 0x0D
    GENDER = 0x02
    MASK = 0x03
    EMOTION = 0x04
    GESTURE = 0x05
    MODEL_TOYS = 0x07
    TRAFFIC_LIGHT_MODEL = 0x08
    GARBAGE = 0x06
    HANDWRITTEN_DIGIT = 0x09
    HANDWRITTEN_LETTER = 0x0A
    COLOR = 0x0B
    CUSTOM_COLOR = 0x0C
    CUSTOM_AI_MODULE = 0x80
    LOCAL_CUSTOM_MODEL = 0x0E


class IdentifyResultType(Enum):
    pass


class TrafficIdentifyResult(IdentifyResultType):
    """交通模型识别结果

    TURN_LEFT: 左转

    TURN_RIGHT: 右转

    NO_LONG_PARKING: 禁止长时间停车

    WHISTLE: 鸣笛

    ENTER_TUNNEL: 进入隧道

    CHILDREN: 注意儿童

    RED_LIGHT: 红灯

    GREEN_LIGHT: 绿灯

    YELLOW_LIGHT: 黄灯

    STOP_LINE: 停止线
    """
    TURN_LEFT = (0x01, 0x00)
    TURN_RIGHT = (0x01, 0x01)
    NO_LONG_PARKING = (0x01, 0x02)
    WHISTLE = (0x01, 0x03)
    ENTER_TUNNEL = (0x01, 0x04)
    CHILDREN = (0x01, 0x05)
    RED_LIGHT = (0x01, 0x06)
    GREEN_LIGHT = (0x01, 0x07)
    YELLOW_LIGHT = (0x01, 0x08)
    STOP_LINE = (0x01, 0x09)


class GenderIdentifyResult(IdentifyResultType):
    """性别识别结果

    MAN: 男

    WOMAN: 女
    """
    MAN = (0x01, 0x00)
    WOMAN = (0x01, 0x01)


class MaskIdentifyResult(IdentifyResultType):
    """口罩识别结果

    NO_WARE_MASK: 没戴口罩

    WARE_MASK: 戴口罩
    """
    NO_WARE_MASK = (0x01, 0x00)
    WARE_MASK = (0x01, 0x02)


class EmotionIdentifyResult(IdentifyResultType):
    """情绪识别结果

    HAPPY: 开心

    CALM: 平静

    ANGRY: 生气

    SURPRISED: 惊讶
    """
    HAPPY = (0x01, 0x00)
    CALM = (0x01, 0x01)
    ANGRY = (0x01, 0x02)
    SURPRISED = (0x01, 0x03)


class GestureIdentifyResult(IdentifyResultType):
    """手势识别结果

    OK: OK

    ROCK: 石头

    SCISSORS: 剪刀

    CLOTH: 布
    """
    OK = (0x01, 0x03)
    ROCK = (0x01, 0x04)
    SCISSORS = (0x01, 0x05)
    CLOTH = (0x01, 0x06)


class ModelToysIdentifyResult(IdentifyResultType):
    """模型小车小人识别结果

    CAR: 模型小车

    ROBOT: 模型小人
    """
    CAR = (0x01, 0x00)
    ROBOT = (0x01, 0x01)


class TrafficLightIdentifyResult(IdentifyResultType):
    """模型红绿灯识别结果

    RED: 模型红灯

    GREEN: 模型绿灯

    YELLOW: 模型黄灯
    """
    RED = (0x01, 0x00)
    GREEN = (0x01, 0x01)
    YELLOW = (0x01, 0x02)


class GarbageIdentifyResult(IdentifyResultType):
    """垃圾识别结果

    BONE: 吃剩的大棒骨

    PAPER: 废纸团

    CANDY: 塑料包装纸

    DURIAN: 榴莲壳

    USED_TISSUE: 用过的纸巾

    BOOK: 废弃书本

    BOTTLE: 塑胶瓶

    CARTON: 纸盒

    CLEAN_BOTTLE: 干净的矿泉水瓶

    METAL_TABLEWARE: 金属餐具

    CHICKEN: 鸡骨头

    KIWI: 猕猴桃皮

    APPLE: 苹果核

    BREAD: 剩饼干

    BANANA: 香蕉皮

    TEA: 碎茶叶

    MELON: 西瓜皮

    FISH: 吃剩的鱼骨头

    CELL: 废旧干电池

    CAPSULE: 过期药片

    BULB: 坏灯泡
    """
    BONE = (0x01, 0x00)
    PAPER = (0x01, 0x01)
    CANDY = (0x01, 0x02)
    DURIAN = (0x01, 0x03)
    USED_TISSUE = (0x01, 0x04)
    BOOK = (0x02, 0x00)
    BOTTLE = (0x02, 0x01)
    CARTON = (0x02, 0x02)
    CLEAN_BOTTLE = (0x02, 0x03)
    METAL_TABLEWARE = (0x02, 0x04)
    CHICKEN = (0x03, 0x00)
    KIWI = (0x03, 0x01)
    APPLE = (0x03, 0x02)
    BREAD = (0x03, 0x03)
    BANANA = (0x03, 0x04)
    TEA = (0x03, 0x05)
    MELON = (0x03, 0x06)
    FISH = (0x03, 0x07)
    CELL = (0x04, 0x00)
    CAPSULE = (0x04, 0x01)
    BULB = (0x04, 0x02)


class ColorIdentifyResult(IdentifyResultType):
    """颜色识别结果

    RED: 红色

    ORANGE: 橙色

    YELLOW: 黄色

    GREEN: 绿色

    BLUE: 蓝色

    PURPLE: 紫色
    """
    RED = (0x01, 0x00)
    ORANGE = (0x01, 0x01)
    YELLOW = (0x01, 0x02)
    GREEN = (0x01, 0x03)
    BLUE = (0x01, 0x04)
    PURPLE = (0x01, 0x05)


class IdentifyType:
    """模型识别结果所属类别"""
    _id: int
    _name: str

    def __init__(self, _id: int, _name: str):
        self._id = _id
        self._name = _name

    def __str__(self, *args, **kwargs):
        return self._name

    def getId(self) -> int:
        """模型识别结果类别标识"""
        return self._id

    def getName(self) -> str:
        """模型识别结果类别名称"""
        return self._name


class IdentifyElement:
    """模型识别结果所属元素"""
    _id: int
    _type: IdentifyResultType
    _name: str

    def __init__(self, _id: int, _type: IdentifyResultType, _name: str):
        self._id = _id
        self._type = _type
        self._name = _name

    def __str__(self, *args, **kwargs):
        return self._name

    def getType(self) -> IdentifyResultType:
        """模型识别结果元素对象定义"""
        return self._type

    def getId(self) -> int:
        """模型识别结果元素标识"""
        return self._id

    def getName(self) -> str:
        """模型识别结果元素名称"""
        return self._name


class IdentifyResult:
    """模型识别结果"""
    _type: IdentifyType
    _element: IdentifyElement
    _x: int
    _y: int
    _w: int
    _h: int

    def __init__(self, _type: IdentifyType, _element: IdentifyElement, _x: int, _y: int, _w: int, _h: int):
        self._type = _type
        self._element = _element
        self._x = _x
        self._y = _y
        self._w = _w
        self._h = _h

    def __str__(self):
        return 'Type: {}, Element: {}, X: {}, Y: {}, Width: {}, Height: {}'.format(self._type, self._element, self._x, self._y, self._w, self._h)

    def getIdentifyType(self) -> IdentifyType:
        """获取模型识别结果所属类别"""
        return self._type

    def isIdentifyResultType(self, t: IdentifyResultType) -> bool:
        """判断是否指定所属类别"""
        return self._element is not None and self._element.getType() == t

    def getIdentifyElement(self) -> IdentifyElement:
        """获取模型识别结果所属元素"""
        return self._element

    def getX(self) -> int:
        """获取模型识别结果区域的左上角位置X坐标"""
        return self._x

    def getCenterX(self) -> int:
        """获取模型识别结果区域的中心位置X坐标"""
        return int(self._x + (self._w / 2))

    def getY(self) -> int:
        """获取模型识别结果区域的左上角位置Y坐标"""
        return self._y

    def getCenterY(self) -> int:
        """获取模型识别结果区域的中心位置Y坐标"""
        return int(self._y + (self._h / 2))

    def getWidth(self) -> int:
        """获取模型识别结果区域的宽度"""
        return self._w

    def getHeight(self) -> int:
        """获取模型识别结果区域的高度"""
        return self._h


def __find_identify_result(model: int, tid: int, eid: int) -> IdentifyResultType:
    if model == VisionModel.TRAFFIC.value:
        for result in TrafficIdentifyResult:
            _tid, _eid = result.value
            if _tid == tid and _eid == eid:
                return result
    elif model == VisionModel.GENDER.value:
        for result in GenderIdentifyResult:
            _tid, _eid = result.value
            if _tid == tid and _eid == eid:
                return result
    elif model == VisionModel.MASK.value:
        for result in MaskIdentifyResult:
            _tid, _eid = result.value
            if _tid == tid and _eid == eid:
                return result
    elif model == VisionModel.EMOTION.value:
        for result in EmotionIdentifyResult:
            _tid, _eid = result.value
            if _tid == tid and _eid == eid:
                return result
    elif model == VisionModel.GESTURE.value:
        for result in GestureIdentifyResult:
            _tid, _eid = result.value
            if _tid == tid and _eid == eid:
                return result
    elif model == VisionModel.MODEL_TOYS.value:
        for result in ModelToysIdentifyResult:
            _tid, _eid = result.value
            if _tid == tid and _eid == eid:
                return result
    elif model == VisionModel.TRAFFIC_LIGHT_MODEL.value:
        for result in TrafficLightIdentifyResult:
            _tid, _eid = result.value
            if _tid == tid and _eid == eid:
                return result
    elif model == VisionModel.GARBAGE.value:
        for result in GarbageIdentifyResult:
            _tid, _eid = result.value
            if _tid == tid and _eid == eid:
                return result
    elif model == VisionModel.COLOR.value:
        for result in ColorIdentifyResult:
            _tid, _eid = result.value
            if _tid == tid and _eid == eid:
                return result
    return None


def __model_identify(model: VisionModel, link: uKitAiLink) -> list:
    if model is None:
        return None
    message = {'model': model.value}
    pbReq = d20_VisionModule_pb2.d20c1008_visionmodule_model_switch_rq()
    pbRsp = d20_VisionModule_pb2.d20c1008_visionmodule_model_switch_ra()
    ack, response = link.sendRequest(CompType.VISION, 1, 1008, message, pbReq, pbRsp, timeout_ms=5500)
    if ack != 0 or response is None:
        return None
    _result = getattr(response, 'result')
    if _result != 0:
        return None
    for i in range(10):
        message = {'model': model.value}
        pbReq = d20_VisionModule_pb2.d20c1005_visionmodule_identify_rq()
        pbRsp = d20_VisionModule_pb2.d20c1005_visionmodule_identify_ra()
        ack, response = link.sendRequest(CompType.VISION, 1, 1005, message, pbReq, pbRsp, timeout_ms=500)
        if ack == 0 and response is not None:
            _model = getattr(response, 'model')
            if model.value != _model:
                time.sleep(0.1)
                continue
            if not hasattr(response, 'info'):
                time.sleep(0.1)
                continue
            info = getattr(response, 'info')
            result = []
            for j in range(len(info)):
                tid = getattr(info[j], 'tid')
                if tid == 0:
                    continue
                eid = getattr(info[j], 'eid')
                tname = getattr(info[j], 'tname')
                ename = getattr(info[j], 'ename')
                x = getattr(info[j], 'x')
                y = getattr(info[j], 'y')
                w = getattr(info[j], 'w')
                h = getattr(info[j], 'h')
                identifyResultType = __find_identify_result(_model, tid, eid)
                _type: IdentifyType = IdentifyType(tid, tname)
                _element: IdentifyElement = IdentifyElement(eid, identifyResultType, ename)
                result.append(IdentifyResult(_type, _element, x, y, w, h))
            if len(result) == 0:
                return None
            return result
        else:
            time.sleep(0.1)
    # 请求失败
    return None


def traffic_identify(*, link: uKitAiLink) -> list:
    """交通标志识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (list): IdentifyResult数组，包含所有识别到的交通标志，接口调用失败则返回None
    """
    return __model_identify(VisionModel.TRAFFIC, link)


def face_identify(*, link: uKitAiLink) -> list:
    """人脸识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (list): IdentifyResult数组，包含所有识别到的人脸信息，接口调用失败则返回None
    """
    return __model_identify(VisionModel.FACE, link)


def face_num_identify(*, link: uKitAiLink) -> int:
    """人脸数量识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (int): 识别到的人脸数量，接口调用失败则返回None
    """
    result = __model_identify(VisionModel.FACE_COUNT, link)
    if result is None:
        return None
    return len(result)


def gender_identify(*, link: uKitAiLink) -> list:
    """性别识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (list): IdentifyResult数组，包含所有识别到的性别信息，接口调用失败则返回None
    """
    return __model_identify(VisionModel.GENDER, link)


def mask_identify(*, link: uKitAiLink) -> list:
    """口罩识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (list): IdentifyResult数组，包含所有识别到的口罩信息，接口调用失败则返回None
    """
    return __model_identify(VisionModel.MASK, link)


def emotion_identify(*, link: uKitAiLink) -> list:
    """情绪识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (list): IdentifyResult数组，包含所有识别到的情绪信息，接口调用失败则返回None
    """
    return __model_identify(VisionModel.EMOTION, link)


def gesture_identify(*, link: uKitAiLink) -> list:
    """手势识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (list): IdentifyResult数组，包含所有识别到的手势信息，接口调用失败则返回None
    """
    return __model_identify(VisionModel.GESTURE, link)


def garbage_identify(*, link: uKitAiLink) -> list:
    """垃圾识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (list): IdentifyResult数组，包含所有识别到的垃圾信息，接口调用失败则返回None
    """
    return __model_identify(VisionModel.GARBAGE, link)


def model_toys_identify(*, link: uKitAiLink) -> list:
    """模型小车小人识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (list): IdentifyResult数组，包含所有识别到的模型小车小人信息，接口调用失败则返回None
    """
    return __model_identify(VisionModel.MODEL_TOYS, link)


def traffic_light_identify(*, link: uKitAiLink) -> list:
    """模型红绿灯识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (list): IdentifyResult数组，包含所有识别到的模型红绿灯信息，接口调用失败则返回None
    """
    return __model_identify(VisionModel.TRAFFIC_LIGHT_MODEL, link)


def handwritten_digit_identify(*, link: uKitAiLink) -> list:
    """手写数字识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (list): IdentifyResult数组，包含所有识别到的手写数字信息，接口调用失败则返回None
    """
    return __model_identify(VisionModel.HANDWRITTEN_DIGIT, link)


def handwritten_letter_identify(*, link: uKitAiLink) -> list:
    """手写字母识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (list): IdentifyResult数组，包含所有识别到的手写字母信息，接口调用失败则返回None
    """
    return __model_identify(VisionModel.HANDWRITTEN_LETTER, link)


def color_identify(*, link: uKitAiLink) -> list:
    """颜色识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (list): IdentifyResult数组，包含所有识别到的颜色信息，接口调用失败则返回None
    """
    return __model_identify(VisionModel.COLOR, link)


def custom_color_identify(*, link: uKitAiLink) -> list:
    """自定义颜色识别

    参数值说明:
        link: 设备连接

    返回值说明:
        (list): IdentifyResult数组，包含所有识别到的自定义颜色信息，接口调用失败则返回None
    """
    return __model_identify(VisionModel.CUSTOM_COLOR, link)


def get_mid_offset(*, link: uKitAiLink) -> int:
    """获取中部偏移量

    参数值说明:
        link: 设备连接

    返回值说明:
        (int): 中部偏移量，接口调用失败则返回None
    """
    message = None
    pbReq = d20_VisionModule_pb2.d20c1003_visionmodule_mid_offset_rq()
    pbRsp = d20_VisionModule_pb2.d20c1003_visionmodule_mid_offset_ra()
    ack, response = link.sendRequest(CompType.VISION, 1, 1003, message, pbReq, pbRsp, timeout_ms=500)
    if ack != 0 or response is None:
        return None
    offset = getattr(response, 'offset')
    return int(offset)



from enum import Enum

from ukitai.link import *
from ukitai.common import *
from ukitai.protos import *
from ukitai.apis import *


class Emotion(Enum):
    """表情定义

    BLINK: 眨眼

    SHY: 害羞

    TEARS: 热泪盈眶

    FLASHING_TEARS: 泪光闪动

    CRY: 哭泣

    DIZZY: 晕

    HAPPY: 开心

    SURPRISED: 惊讶

    BREATH: 呼吸

    FLASH: 闪烁

    FAN: 风扇

    WIPERS: 雨刮
    """
    BLINK = 0
    SHY = 1
    TEARS = 2
    FLASHING_TEARS = 3
    CRY = 4
    DIZZY = 5
    HAPPY = 6
    SURPRISED = 7
    BREATH = 8
    FLASH = 9
    FAN = 10
    WIPERS = 11


class Scene(Enum):
    """场景定义

    COLORED_LIGHTS: 七彩跑马灯

    DISCO: Disco

    PRIMARY_COLOR: 三原色

    COLOR_STACKING: 色彩堆叠
    """
    COLORED_LIGHTS = 12
    DISCO = 13
    PRIMARY_COLOR = 14
    COLOR_STACKING = 15


def set_state(*, id: int, enabled: bool, link: uKitAiLink):
    """打开/关闭眼灯功能

    参数值说明:
        id: 传感器 id

        enabled: True=开打眼灯功能，False=关闭眼灯功能

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    return CommonApi.comp_state(compType=CompType.LED, id=id, enabled=enabled, link=link)


def show_emotion(*, id: int, emotion: Emotion, color: Color, times: int, link: uKitAiLink):
    """显示表情控制

    参数值说明:
        id: 传感器 id

        emotion (Emotion): 表情id

        color (Color): 颜色值

        times (int): 重复次数

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    if color:
        color = (color.color << 8) & 0xFFFFFF00
    else:
        color = 0
    times = int(Common.num_normal(times, 0xFFFF, 0))
    message = {'expressions_type': emotion.value, 'time': times, 'rgbc': color}
    pbReq = d8_Led_pb2.d8c1000_led_fix_exp_set_rq()
    pbRsp = d8_Led_pb2.d8c1000_led_fix_exp_set_ra()
    return link.sendRequest(CompType.LED, id, 1000, message, pbReq, pbRsp)


def show_scene(*, id: int, scene: Scene, times: int, link: uKitAiLink):
    """显示场景效果

    参数值说明:
        id: 传感器 id

        scene (Scene): 场景id

        times (int): 重复次数

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    times = int(Common.num_normal(times, 0xFFFF, 0))
    message = {'expressions_type': scene.value, 'time': times, 'rgbc': 0}
    pbReq = d8_Led_pb2.d8c1000_led_fix_exp_set_rq()
    pbRsp = d8_Led_pb2.d8c1000_led_fix_exp_set_ra()
    return link.sendRequest(CompType.LED, id, 1000, message, pbReq, pbRsp)


def show_custom_light(*, id: int, colors: list, time: int, link: uKitAiLink):
    """自定义灯光颜色

    参数值说明:
        id: 传感器 id

        colors (list): 每一个灯瓣颜色，长度为8

        time (int): 亮灯时间

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    time = int(Common.num_normal(time, 0xFFFFFFFF, 0))
    color_lump = []
    for i in range(len(colors)):
        color_lump.append({'index': 1 << i, 'rgbc': (colors[i].color << 8) & 0xFFFFFF00})
    message = {'color_lump': color_lump, 'time': time}
    pbReq = d8_Led_pb2.d8c1001_led_exp_set_rq()
    pbRsp = d8_Led_pb2.d8c1001_led_exp_set_ra()
    return link.sendRequest(CompType.LED, id, 1001, message, pbReq, pbRsp)


def turn_off(*, id: int, link: uKitAiLink):
    """关闭灯光

    参数值说明:
        id: 传感器 id

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'expressions_type': 0, 'time': 0, 'rgbc': 0}
    pbReq = d8_Led_pb2.d8c1000_led_fix_exp_set_rq()
    pbRsp = d8_Led_pb2.d8c1000_led_fix_exp_set_ra()
    return link.sendRequest(CompType.LED, id, 1000, message, pbReq, pbRsp)



from enum import Enum

from ukitai.apis import *
from ukitai.common import *
from ukitai.link import *
from ukitai.protos import *


class ServoDirection(Enum):
    """运动方向

    CLOCKWISE: 顺时针

    ANTICLOCKWISE: 逆时针

    STOP: 停止
    """
    CLOCKWISE = 1
    ANTICLOCKWISE = -1
    STOP = 0


class SpeedMode(Enum):
    """速度档位

    VERY_SLOW: 非常慢

    SLOW: 慢

    MEDIUM: 中速

    FAST: 快

    VERY_FAST: 非常快
    """
    VERY_SLOW = 128
    SLOW = 234
    MEDIUM = 500
    FAST = 800
    VERY_FAST = 1000


def read_servo_angle(*, id: int, link: uKitAiLink):
    """读取舵机角度

    参数值说明:
        id: 舵机 id

        link: 设备连接

    返回值说明:
        (int): 舵机角度，范围[-120, 120]，读取失败时返回None
    """
    return CommonApi.read_value(id=id, data_type=DataType.SERVO_ANGLE, link=link)


def turn_servo_angle(*, id: int, angle: int, duration_ms: int, link: uKitAiLink) -> tuple:
    """控制舵机用指定的时长转转至某个角度

    参数值说明:
        id: 舵机 id

        angle (int): 指定舵机旋转到达的角度值,单位:度, 范围:[-118, 118]

        duration_ms (int): 旋转耗时,单位:毫秒,范围:[20, 5000]

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据

    示例:
        # 控制 id 为 2 的舵机用 500 毫秒转动到 100°

        Motion.turn_servo_angle(id=2, angle=100, duration_ms=500, link=link)

        # 控制 id 为 1 的舵机用 1 秒转动到 -80°

        Motion.turn_servo_angle(id=1, angle=-80, duration_ms=1000, link=link)
    """
    duration = int(Common.num_normal(duration_ms, 5000, 20))
    angle = int(Common.num_normal(angle + 120, 238, 2) * 10)
    message = {'tar_angle': angle, 'rotation_time': duration}
    pbReq = d3_Servo_pb2.d3c1000_srv_angle_set_rq()
    pbRsp = d3_Servo_pb2.d3c1000_srv_angle_set_ra()
    return link.sendRequest(CompType.SERVO, id, 1000, message, pbReq, pbRsp)


def turn_servo_speed_mode(*, id: int, direction: ServoDirection, speed_mode: SpeedMode, link: uKitAiLink):
    """控制舵机按照指定的方向和速度档位转动

    参数值说明:
        id: 舵机 id
        direction (Servo.ServoDirection): 指定转动方向, 如果参数为 Servo.ServoDirection.STOP, 舵机会停止
        speed_mode (Servo.SpeedMode): 速度档位
        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据

    示例:
        # 控制 id 为 2 的舵机以非常慢的速度逆时针转动

        Servo.turn_servo_speed_mode(id=2, direction=Servo.ServoDirection.ANTICLOCKWISE, speed_mode=Servo.SpeedMode.VERY_SLOW, link=link)

        # 停止舵机 1

        Servo.turn_servo_speed_mode(id=1, direction=Servo.ServoDirection.STOP, speed_mode=Servo.SpeedMode.VERY_SLOW, link=link)
    """
    speed = speed_mode.value * direction.value
    message = {'pwm': speed}
    pbReq = d3_Servo_pb2.d3c1002_srv_pwm_set_rq()
    pbRsp = d3_Servo_pb2.d3c1002_srv_pwm_set_ra()
    return link.sendRequest(CompType.SERVO, id, 1002, message, pbReq, pbRsp)


def turn_servo_speed(*, id: int, direction: ServoDirection, speed: int, link: uKitAiLink):
    """控制舵机按照指定的方向和速度旋转

    参数值说明:
        id: 舵机 id
        direction (Servo.ServoDirection): 指定转动方向, 如果参数为 Servo.ServoDirection.STOP, 舵机会停止
        speed (int): 速度, 单位: 度/秒, 范围:[0, 1000]
        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据

    示例:
        # 控制 id 为 2 的舵机以100 度/秒的速度逆时针转动

        Servo.turn_servo_speed(id=2, direction=Servo.ServoDirection.ANTICLOCKWISE, speed=100, link=link)

        # 停止舵机 1

        Servo.turn_servo_speed(id=1, direction=Servo.ServoDirection.STOP, speed=0, link=link)
    """
    speed = int(Common.num_normal(speed, 1000, 0)) * direction.value
    message = {'pwm': speed}
    pbReq = d3_Servo_pb2.d3c1002_srv_pwm_set_rq()
    pbRsp = d3_Servo_pb2.d3c1002_srv_pwm_set_ra()
    return link.sendRequest(CompType.SERVO, id, 1002, message, pbReq, pbRsp)


def turn_servo_speed_percent(*, id: int, direction: ServoDirection, speed_percent: int, link: uKitAiLink):
    """控制舵机按照指定的方向和速度百分比旋转

    参数值说明:
        id: 舵机 id
        direction (Servo.ServoDirection): 指定转动方向, 如果参数为 Servo.ServoDirection.STOP, 舵机会停止
        speed_percent (int): 全速 658 度/秒 的百分比, 范围:[0, 100]
        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据


    示例:
        # 控制 id 为 1 的舵机以全速 50% 的速度顺时针转动

        Servo.turn_servo_speed_percent(id=1, direction=Servo.ServoDirection.CLOCKWISE, speed_percent=50, link=link)

        # 控制 id 为 2 的舵机以全速逆时针转动

        Servo.turn_servo_speed_percent(id=2, direction=Servo.ServoDirection.ANTICLOCKWISE, speed_percent=100, link=link)
    """
    speed = int(Common.num_normal(speed_percent, 100, 0)) * 10 * direction.value
    message = {'pwm': speed}
    pbReq = d3_Servo_pb2.d3c1002_srv_pwm_set_rq()
    pbRsp = d3_Servo_pb2.d3c1002_srv_pwm_set_ra()
    return link.sendRequest(CompType.SERVO, id, 1002, message, pbReq, pbRsp)


def stop_servo(*, id: int, mode: int = 0, link: uKitAiLink):
    """控制舵机停止转动

    参数值说明:
        id: 舵机 id
        mode: 保留字段，默认为0

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    mode = int(Common.num_normal(mode, 3, 0))
    message = {'mode': mode}
    pbReq = d3_Servo_pb2.d3c1004_srv_stop_rq()
    pbRsp = d3_Servo_pb2.d3c1004_srv_stop_ra()
    return link.sendRequest(CompType.SERVO, id, 1004, message, pbReq, pbRsp)



import time
from enum import Enum

from ukitai.apis import *
from ukitai.common import *
from ukitai.link import *
from ukitai.protos import *


class Tone(Enum):
    """音调

    C5, D5, E5, F5, G5, A5, B5, C6
    """
    C5 = 'C5'
    D5 = 'D5'
    E5 = 'E5'
    F5 = 'F5'
    G5 = 'G5'
    A5 = 'A5'
    B5 = 'B5'
    C6 = 'C6'


class Beat(Enum):
    """节拍

    EIGHTH: 1/8拍

    QUARTER: 1/4拍

    HALF: 1/2拍

    ONE: 1拍

    DOUBLE: 2拍
    """
    EIGHTH = '-1-8'
    QUARTER = '-1-4'
    HALF = '-1-2'
    ONE = ''
    DOUBLE = '-2'


class Volume(Enum):
    """音量等级

    MINIMUM: 最低音量

    LOW: 低音量

    MEDIUM: 中等音量

    HIGH: 高音量

    MAXIMUM: 最高音量
    """
    MINIMUM = 42
    LOW = 56
    MEDIUM = 70
    HIGH = 84
    MAXIMUM = 98


class Sound(Enum):
    pass


class Animal(Sound):
    """动物声音

    ELEPHANT: 大象

    BEAR: 熊

    BIRD: 鸟

    CHICKEN: 鸡

    COW: 牛

    DOG: 狗

    GIRAFFE: 长颈鹿

    HORSE: 马

    LION: 狮子

    MONKEY: 猴子

    PIG: 猪

    RHINO: 犀牛

    SEA_LION: 海狮

    TIGER: 老虎

    WALRUS: 海象
    """
    __AT = 'animal'
    ELEPHANT = (__AT, 'elephant')
    BEAR = (__AT, 'bear')
    BIRD = (__AT, 'bird')
    CHICKEN = (__AT, 'chicken')
    COW = (__AT, 'cow')
    DOG = (__AT, 'dog')
    GIRAFFE = (__AT, 'giraffe')
    HORSE = (__AT, 'horse')
    LION = (__AT, 'lion')
    MONKEY = (__AT, 'monkey')
    PIG = (__AT, 'pig')
    RHINO = (__AT, 'rhinoceros')
    SEA_LION = (__AT, 'sealions')
    TIGER = (__AT, 'tiger')
    WALRUS = (__AT, 'walrus')


class Machine(Sound):
    """机器声音

    AMBULANCE: 救护车

    BUSY: 忙音

    CAR_HORN1: 汽车喇叭1

    CAR_HORN2: 汽车喇叭2

    DOORBELL: 门铃

    ENGINE: 引擎

    LASER: 激光

    MEEBOT: 小黄人

    POLICE_CAR1: 警车1

    POLICE_CAR2: 警车2

    INCOMING_CALL: 来电铃声

    ROBOT: 机器人

    OUTGOING_CALL: 电话呼叫

    TOUCH_TONE: 按键音

    WAVE: 电波
    """
    __MT = 'machine'
    AMBULANCE = (__MT, 'ambulance')
    BUSY = (__MT, 'busy_tone')
    CAR_HORN1 = (__MT, 'carhorn')
    CAR_HORN2 = (__MT, 'carhorn1')
    DOORBELL = (__MT, 'doorbell')
    ENGINE = (__MT, 'engine')
    LASER = (__MT, 'laser')
    MEEBOT = (__MT, 'meebot')
    POLICE_CAR1 = (__MT, 'police_car_1')
    POLICE_CAR2 = (__MT, 'police_car_2')
    INCOMING_CALL = (__MT, 'ringtones')
    ROBOT = (__MT, 'robot')
    OUTGOING_CALL = (__MT, 'telephone_call')
    TOUCH_TONE = (__MT, 'touch_tone')
    WAVE = (__MT, 'wave')


class Emotion(Sound):
    """情绪声音

    HAPPY: 高兴

    SURPRISED: 惊讶

    CHEERFUL: 愉快

    TEARS: 热泪盈眶

    SOLILOQUY: 呓语

    SNORING: 呼噜

    YAWN: 哈欠

    DOUBT: 疑问

    ANGRY: 生气

    UPSET: 失落

    FRUSTRATED: 失败

    MUSIC1: 歌曲1

    MUSIC2: 歌曲2

    MUSIC3: 歌曲3

    MUSIC4: 歌曲4
    """
    __ET = 'emotion'
    HAPPY = (__ET, 'happy')
    SURPRISED = (__ET, 'surprise')
    CHEERFUL = (__ET, 'cheerful')
    TEARS = (__ET, 'actingcute')
    SOLILOQUY = (__ET, 'nonsense')
    SNORING = (__ET, 'snoring')
    YAWN = (__ET, 'yawn')
    DOUBT = (__ET, 'doubt')
    ANGRY = (__ET, 'angry')
    UPSET = (__ET, 'lose')
    FRUSTRATED = (__ET, 'fail')
    MUSIC1 = (__ET, 'come_and_play')
    MUSIC2 = (__ET, 'flexin')
    MUSIC3 = (__ET, 'london_bridge')
    MUSIC4 = (__ET, 'yankee_doodle')


class Command(Sound):
    """命令声音

    OBEY: 遵命

    RECEIVED: 收到

    COMPLETE: 完成

    TRANSFORM: 变身

    COVER: 掩护

    SUPPORT: 支援

    MOVE: 移动
    """
    __CT = 'command'
    OBEY = (__CT, 'yes')
    RECEIVED = (__CT, 'received')
    COMPLETE = (__CT, 'complete')
    TRANSFORM = (__CT, 'transfiguration')
    COVER = (__CT, 'cover')
    SUPPORT = (__CT, 'support')
    MOVE = (__CT, 'move')


def play_tone(*, tone: Tone, beat: Beat, link: uKitAiLink):
    """控制主控以指定节拍播放指定音调

    参数值说明:
        tone: 音调, 定义在 Audio.Tone 中

        beat: 节拍, 定义在 Audio.Beat 中

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据

    示例:
        # 以 1/2 节拍播放 C6 音调

        Audio.play_tone(tone=Audio.Tone.C6, beat=Audio.Beat.HALF, link=link)

        # 以 1 节拍播放 C5 音调

        Audio.play_tone(tone=Audio.Tone.C5, beat=Audio.Beat.ONE, link=link)
    """
    file_name = tone.value + beat.value
    message = {'file_type': 'tune', 'file_name': file_name}
    pbReq = dE_VoiceRecognition_pb2.dec1026_vr_effect_play_start_rq()
    pbRsp = dE_VoiceRecognition_pb2.dec1026_vr_effect_play_start_ra()
    return link.sendRequest(CompType.AUDIO, 0, 1026, message, pbReq, pbRsp)


def play_sound(*, sound: Sound, link: uKitAiLink):
    """控制主控以指定节拍播放指定音调

    参数值说明:
        sound: 音效, 定义在 Audio.Sound 中

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据

    示例:
        # 以 1/2 节拍播放 C6 音调

        Audio.play_sound(sound=Audio.Animal.ELEPHANT, link=link)

        # 以 1 节拍播放 C5 音调

        Audio.play_sound(sound=Audio.Emotion.CHEERFUL, link=link)
    """
    _type, name = sound.value
    message = {'file_type': _type, 'file_name': name}
    pbReq = dE_VoiceRecognition_pb2.dec1026_vr_effect_play_start_rq()
    pbRsp = dE_VoiceRecognition_pb2.dec1026_vr_effect_play_start_ra()
    return link.sendRequest(CompType.AUDIO, 0, 1026, message, pbReq, pbRsp)


def play_record(*, name: str, link: uKitAiLink):
    """控制主控以指定节拍播放指定音调

    参数值说明:
        name: 录音名称

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据

    示例:
        # 播放录音 "录音文件1", 不等待播放结束

        Audio.play_record(name="录音文件1", link=link)

        # 播放录音 "aaa"

        Audio.play_record(name="aaa", link=link)
    """
    message = {'file_name': name}
    pbReq = dE_VoiceRecognition_pb2.dec1027_vr_record_play_start_rq()
    pbRsp = dE_VoiceRecognition_pb2.dec1027_vr_record_play_start_ra()
    return link.sendRequest(CompType.AUDIO, 0, 1027, message, pbReq, pbRsp)


def read_record_nums(*, link: uKitAiLink):
    """获取录音文件个数

    参数值说明:
        link: 设备连接

    返回值说明:
        (int): 录音文件个数，读取失败时返回None
    """
    message = None
    pbReq = dE_VoiceRecognition_pb2.dec1039_vr_file_num_rq()
    pbRsp = dE_VoiceRecognition_pb2.dec1039_vr_file_num_ra()
    ack, response = link.sendRequest(CompType.AUDIO, 0, 1039, message, pbReq, pbRsp)
    if ack != 0 or not response:
        # 请求失败
        return None
    file_num = getattr(response, 'file_num')
    return file_num


def read_records(*, link: uKitAiLink) -> dict:
    """获取录音列表

    参数值说明:
        link: 设备连接

    返回值说明:
        (dist): 录音文件列表以及对应的录音时长，读取失败时返回None
    """
    message = None
    pbReq = dE_VoiceRecognition_pb2.dec1022_vr_file_list_rq()
    pbRsp = dE_VoiceRecognition_pb2.dec1022_vr_file_list_ra()
    ack, response = link.sendRequest(CompType.AUDIO, 0, 1022, message, pbReq, pbRsp)
    if ack != 0 or not response:
        # 请求失败
        return None
    _list = getattr(response, 'list')
    result = {}
    for i in range(len(_list)):
        result[getattr(_list[i], 'file_name')] = getattr(_list[i], 'file_duration')
    return result


def stop_play(*, link: uKitAiLink) -> dict:
    """停止播放声音

    参数值说明:
        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = None
    pbReq = dE_VoiceRecognition_pb2.dec1028_vr_file_play_stop_rq()
    pbRsp = dE_VoiceRecognition_pb2.dec1028_vr_file_play_stop_ra()
    return link.sendRequest(CompType.AUDIO, 0, 1028, message, pbReq, pbRsp)


def play_tts(*, content: str, link: uKitAiLink):
    """停止播放声音

    参数值说明:
        content: 播放文本内容

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'text': content, 'language': 'zh'}
    pbReq = dE_VoiceRecognition_pb2.dec1030_vr_tts_play_rq()
    pbRsp = dE_VoiceRecognition_pb2.dec1030_vr_tts_play_rq()
    return link.sendRequest(CompType.AUDIO, 0, 1030, message, pbReq, pbRsp)


def set_volume(*, volume: Volume, link: uKitAiLink):
    """设置音量等级

    参数值说明:
        volume: 音量等级

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'value': volume.value}
    pbReq = dE_VoiceRecognition_pb2.dec1035_vr_set_volume_rq()
    pbRsp = dE_VoiceRecognition_pb2.dec1035_vr_set_volume_ra()
    return link.sendRequest(CompType.AUDIO, 0, 1035, message, pbReq, pbRsp)


def read_volume(*, link: uKitAiLink):
    """读取当前音量等级

    参数值说明:
        link: 设备连接

    返回值说明:
        (int): 当前音量等级，读取失败时返回None
    """
    message = None
    pbReq = dE_VoiceRecognition_pb2.dec1036_vr_get_volume_rq()
    pbRsp = dE_VoiceRecognition_pb2.dec1036_vr_get_volume_ra()
    ack, response = link.sendRequest(CompType.AUDIO, 0, 1036, message, pbReq, pbRsp)
    if ack != 0 or not response:
        # 请求失败
        return None
    volume = getattr(response, 'value')
    return volume


def read_voice_direction(*, link: uKitAiLink):
    """读取当前声源方位

    参数值说明:
        link: 设备连接

    返回值说明:
        (int): 当前声源方位

        None: 读取失败

        0: 没有声音或者超出有效范围

        1: 中间，声源方位角在区间范围 [75, 105]

        2: 左边，声源方位角在区间范围 [0, 75)

        3: 右边，声源方位角在区间范围 (105, 180]
    """
    message = None
    pbReq = dE_VoiceRecognition_pb2.dec1042_vr_dir_init_rq()
    pbRsp = dE_VoiceRecognition_pb2.dec1042_vr_dir_init_ra()
    ack, response = link.sendRequest(CompType.AUDIO, 0, 1042, message, pbReq, pbRsp)
    if ack != 0:
        # 请求失败
        return None
    for i in range(5):
        message = None
        pbReq = dE_VoiceRecognition_pb2.dec1043_vr_dir_get_rq()
        pbRsp = dE_VoiceRecognition_pb2.dec1043_vr_dir_get_ra()
        ack, response = link.sendRequest(CompType.AUDIO, 0, 1043, message, pbReq, pbRsp)
        if ack == 0 and response is not None:
            _dir = getattr(response, 'dir')
            return _dir
        else:
            time.sleep(0.2)
    # 请求失败
    return None


def read_asr(*, link: uKitAiLink):
    """读取asr结果

    参数值说明:
        link: 设备连接

    返回值说明:
        (str): 当前听到的asr结果，读取失败时返回None
    """
    try:
        message = {'language': 'zh'}
        pbReq = dE_VoiceRecognition_pb2.dec1032_vr_asr_start_rq()
        pbRsp = dE_VoiceRecognition_pb2.dec1032_vr_asr_start_ra()
        ack, response = link.sendRequest(CompType.AUDIO, 0, 1032, message, pbReq, pbRsp)
        if ack != 0:
            # 请求失败
            return None
        for i in range(10):
            message = None
            pbReq = dE_VoiceRecognition_pb2.dec1044_vr_asr_get_text_rq()
            pbRsp = dE_VoiceRecognition_pb2.dec1044_vr_asr_get_text_ra()
            ack, response = link.sendRequest(CompType.AUDIO, 0, 1044, message, pbReq, pbRsp)
            if ack == 0 and response is not None:
                text = getattr(response, 'text')
                return text
            else:
                time.sleep(0.5)
        # 请求失败
        return None
    finally:
        message = None
        pbReq = dE_VoiceRecognition_pb2.dec1033_vr_asr_stop_rq()
        pbRsp = dE_VoiceRecognition_pb2.dec1033_vr_asr_stop_ra()
        ack, response = link.sendRequest(CompType.AUDIO, 0, 1033, message, pbReq, pbRsp)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: d8_Led.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x64\x38_Led.proto\"R\n\x1a\x64\x38\x63\x31\x30\x30\x30_led_fix_exp_set_rq\x12\x18\n\x10\x65xpressions_type\x18\x01 \x01(\r\x12\x0c\n\x04time\x18\x02 \x01(\r\x12\x0c\n\x04rgbc\x18\x03 \x01(\r\"\x1c\n\x1a\x64\x38\x63\x31\x30\x30\x30_led_fix_exp_set_ra\"\x91\x01\n\x16\x64\x38\x63\x31\x30\x30\x31_led_exp_set_rq\x12\x0c\n\x04time\x18\x01 \x01(\r\x12:\n\ncolor_lump\x18\x02 \x03(\x0b\x32&.d8c1001_led_exp_set_rq.dev_color_lump\x1a-\n\x0e\x64\x65v_color_lump\x12\r\n\x05index\x18\x01 \x01(\r\x12\x0c\n\x04rgbc\x18\x02 \x01(\r\"\x18\n\x16\x64\x38\x63\x31\x30\x30\x31_led_exp_set_rab\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'd8_Led_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _D8C1000_LED_FIX_EXP_SET_RQ._serialized_start=16
  _D8C1000_LED_FIX_EXP_SET_RQ._serialized_end=98
  _D8C1000_LED_FIX_EXP_SET_RA._serialized_start=100
  _D8C1000_LED_FIX_EXP_SET_RA._serialized_end=128
  _D8C1001_LED_EXP_SET_RQ._serialized_start=131
  _D8C1001_LED_EXP_SET_RQ._serialized_end=276
  _D8C1001_LED_EXP_SET_RQ_DEV_COLOR_LUMP._serialized_start=231
  _D8C1001_LED_EXP_SET_RQ_DEV_COLOR_LUMP._serialized_end=276
  _D8C1001_LED_EXP_SET_RA._serialized_start=278
  _D8C1001_LED_EXP_SET_RA._serialized_end=302
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: header.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cheader.proto\"\xd2\x01\n\x06Header\x12\x0b\n\x03\x64\x65v\x18\x01 \x01(\r\x12\x0b\n\x03\x63md\x18\x02 \x01(\r\x12\n\n\x02id\x18\x03 \x01(\r\x12\x0f\n\x07\x64\x61taLen\x18\x04 \x01(\r\x12\x1a\n\x04\x61ttr\x18\x05 \x01(\x0e\x32\x0c.Header.Attr\x12\x0b\n\x03\x61\x63k\x18\x06 \x01(\x11\x12\x0b\n\x03seq\x18\x07 \x01(\r\"[\n\x04\x41ttr\x12\x0b\n\x07RESERVE\x10\x00\x12\x0b\n\x07REQUEST\x10\x01\x12\n\n\x06REQACK\x10\x02\x12\x0b\n\x07PUBLISH\x10\x03\x12\n\n\x06PUBACK\x10\x04\x12\n\n\x06NOTIFY\x10\x05\x12\x08\n\x04PUSH\x10\x06\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'header_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _HEADER._serialized_start=17
  _HEADER._serialized_end=227
  _HEADER_ATTR._serialized_start=136
  _HEADER_ATTR._serialized_end=227
# @@protoc_insertion_point(module_scope)



from ukitai.link import *
from ukitai.common import *
from ukitai.protos import *
from ukitai.apis import *


class Scene(object):
    """场景定义

    COLORED_LIGHTS: 跑马灯

    DISCO: Disco

    PRIMARY_COLOR: 三原色

    COLOR_STACKING: 色彩堆叠
    """
    COLORED_LIGHTS = 0
    DISCO = 1
    PRIMARY_COLOR = 2
    COLOR_STACKING = 3


def set_lights_brightness(*, id: int, brightness: dict, link: uKitAiLink):
    """设置灯带亮度

    参数值说明:
        id: 灯盒 id

        brightness (dict): 亮度参数, 型如 {belt_id: brightness}

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    _brightness = []
    for k, v in brightness.items():
        _brightness.append({'port': int(k), 'brightness': int(v)})
    message = {'brightness': _brightness}
    pbReq = d1A_LedBelt_pb2.d1ac1003_led_belt_brightness_set_rq()
    pbRsp = d1A_LedBelt_pb2.d1ac1003_led_belt_brightness_set_ra()
    return link.sendRequest(CompType.LIGHT_BOX, id, 1003, message, pbReq, pbRsp)


def show_colors(*, id: int, colors: dict, link: uKitAiLink):
    """控制灯带显示颜色

    参数值说明:
        id: 灯盒 id

        colors (dict): 颜色参数, 型如 {belt_id: (start_beads, end_beads, color)}

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    pixel = []
    for k, (s, e, c) in colors.items():
        s = Common.num_normal(s, 90, 1)
        e = Common.num_normal(e, 90, 1)
        color = 0
        if c:
            color = (c.color << 8) & 0xFFFFFF00
        pixel.append({'port': int(k), 'start_pixel': s, 'end_pixel': e, 'rgbc': color})
    message = {'pixel': pixel}
    pbReq = d1A_LedBelt_pb2.d1ac1006_led_belt_expressions_continuous_set_rq()
    pbRsp = d1A_LedBelt_pb2.d1ac1006_led_belt_expressions_continuous_set_ra()
    return link.sendRequest(CompType.LIGHT_BOX, id, 1006, message, pbReq, pbRsp)


def show_colors_breath(*, id: int, colors: dict, link: uKitAiLink):
    """控制灯带显示呼吸灯效果颜色

    参数值说明:
        id: 灯盒 id

        colors (dict): 颜色参数, 型如 {belt_id: (start_beads, end_beads, color, time=2000)}

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    pixel = []
    for k, v in colors.items():
        if len(v) < 3:
            continue
        s = v[0]
        e = v[1]
        c = v[2]
        if len(v) >= 4:
            t = v[3]
        else:
            t = 2000
        s = Common.num_normal(s, 90, 1)
        e = Common.num_normal(e, 90, 1)
        color = 0
        if c:
            color = (c.color << 8) & 0xFFFFFF00
        pixel.append({'port': int(k), 'start_pixel': s, 'end_pixel': e, 'rgbc': color, 'time': t})
    message = {'pixel': pixel}
    pbReq = d1A_LedBelt_pb2.d1ac1007_led_belt_expressions_continuous_breath_set_rq()
    pbRsp = d1A_LedBelt_pb2.d1ac1007_led_belt_expressions_continuous_breath_set_ra()
    return link.sendRequest(CompType.LIGHT_BOX, id, 1007, message, pbReq, pbRsp)


def move_beads(*, id: int, move_parameters: dict, link: uKitAiLink):
    """控制灯珠移动效果

    参数值说明:
        id: 灯盒 id

        move_parameters (dict): 移动参数, 型如 {belt_id: (count, times)}

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    move = []
    for k, (c, t) in move_parameters.items():
        move.append({'port': int(k), 'pixel': c, 'time': t})
    message = {'move': move}
    pbReq = d1A_LedBelt_pb2.d1ac1005_led_belt_move_set_rq()
    pbRsp = d1A_LedBelt_pb2.d1ac1005_led_belt_move_set_ra()
    return link.sendRequest(CompType.LIGHT_BOX, id, 1005, message, pbReq, pbRsp)


def show_scene(*, id: int, expressions_type: Scene, times: int, color: Color, port: int, link: uKitAiLink):
    """控制灯带显示场景效果

    参数值说明:
        id: 灯盒 id

        expressions_type (Scene): 表情种类

        times (int): 显示次数

        color (Color): 颜色值

        port (int): belt_id

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    _color = 0
    if color:
        _color = (color.color << 8) & 0xFFFFFF00
    message = {'expressions_type': expressions_type, 'time': times, 'port': port, 'rgbc': _color}
    pbReq = d1A_LedBelt_pb2.d1ac1002_led_belt_fix_exp_set_rq()
    pbRsp = d1A_LedBelt_pb2.d1ac1002_led_belt_fix_exp_set_ra()
    return link.sendRequest(CompType.LIGHT_BOX, id, 1002, message, pbReq, pbRsp)


def turn_off(*, id: int, belts: list, link: uKitAiLink):
    """关闭LED灯盒

    参数值说明:
        id: 灯盒 id

        belts (list): 灯盒端口

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'port': belts}
    pbReq = d1A_LedBelt_pb2.d1ac1004_led_belt_off_set_rq()
    pbRsp = d1A_LedBelt_pb2.d1ac1004_led_belt_off_set_ra()
    return link.sendRequest(CompType.LIGHT_BOX, id, 1004, message, pbReq, pbRsp)


def read_blet_nums(*, id: int, link: uKitAiLink):
    """获取灯珠个数

    参数值说明:
        id: 灯盒 id

        link: 设备连接

    返回值说明:
        (dict): 灯盒下各个端口对应的灯珠个数，读取失败时返回None
    """
    message = None
    pbReq = d1A_LedBelt_pb2.d1ac1001_led_belt_leds_num_get_rq()
    pbRsp = d1A_LedBelt_pb2.d1ac1001_led_belt_leds_num_get_ra()
    ack, response = link.sendRequest(CompType.LIGHT_BOX, id, 1001, message, pbReq, pbRsp)
    if ack != 0 or not response:
        # 请求失败
        return None
    num_lump = getattr(response, 'num_lump')
    result = {}
    for i in range(4):
        result[i + 1] = 0
    for i in range(len(num_lump)):
        result[getattr(num_lump[i], 'port')] = getattr(num_lump[i], 'total')
    return result





from ukitai.link import *
from ukitai.common import *
from ukitai.apis import *


def read_battery_level(*, link: uKitAiLink):
    """读取电池电量等级

    参数值说明:
        link: 设备连接

    返回值说明:
        (int): 电量等级，范围[0, 100]，读取失败时返回None
    """
    return CommonApi.read_value(data_type=DataType.BATTERY_LEVEL, link=link)


def read_battery_status(*, link: uKitAiLink):
    """读取电池状态

    参数值说明:
        link: 设备连接

    返回值说明:
        (int):电池状态

        None: 异常

        0: 正常

        1: 低电

        2: 充电中

        3: 充满电

        4: 异常
    """
    return CommonApi.read_value(data_type=DataType.BATTERY_STATUS, link=link)


def read_charge_status(*, link: uKitAiLink):
    """读取适配器状态

    参数值说明:
        link: 设备连接

    返回值说明:
        (int):适配器状态

        None: 异常

        0: 无外接适配器

        1: 有外接适配器
    """
    return CommonApi.read_value(data_type=DataType.CHARGE_STATUS, link=link)



"""定义外设类型 """
INFRARED = 0x5
TOUCH = 0xb
ULTRASOUND = 0x9
COLOR = 0xd
HUMITURE = 0x6
ENV_LIGHT = 0x11
SOUND = 0x7
SERVO = 0x3
MOTOR = 0x4
LED = 0x8
LIGHT_BOX = 0x1a
BATTERY = 0xf
AUDIO = 0xe
LCD = 0x1b
VISION = 0x20
INTELLIGENT_DEVICE = 0x2



import queue
import threading
import time

from ukitai.link.LinkInterface import LinkInterface
from ukitai.protos import *


class uKitAiLink(LinkInterface):
    _queue = None
    _seq = 0
    _send_lock = None
    _receive_lock = None
    _data = bytes([])
    _debug = False

    def __init__(self):
        self._queue = queue.Queue()
        self._send_lock = threading.RLock()
        self._receive_lock = threading.RLock()

    def debug(self, enabled: bool):
        self._debug = enabled
        self._updateDebug(enabled)

    def _updateDebug(self, enabled: bool):
        pass

    def _log(self, msg):
        if not self._debug:
            return
        ct = time.time()
        local_time = time.localtime(ct)
        data_head = time.strftime("%Y-%m-%d %H:%M:%S", local_time)
        data_secs = (ct - int(ct)) * 1000
        print("[%s.%03d]\n%s" % (data_head, data_secs, msg))

    def _dump(self, data: bytes, tag=None):
        if not self._debug:
            return
        if tag:
            sb = str(tag) + "\n"
        else:
            sb = ''
        for i in range(len(data)):
            if i != 0 and i % 16 == 0:
                sb = sb + "\n"
            sb = sb + "%02X " % data[i]
        sb = sb + "\n"
        self._log(sb)

    def _bind_message(self, target, message: dict = None):
        if message and target:
            for k, v in message.items():
                attr = getattr(target, k)
                if type(attr).__name__ == 'RepeatedCompositeContainer' and isinstance(v, list):
                    for item in v:
                        newSubItem = attr.add()
                        self._bind_message(newSubItem, item)
                elif type(attr).__name__ == 'RepeatedScalarContainer' and isinstance(v, list):
                    for item in v:
                        attr.append(item)
                else:
                    try:
                        setattr(target, k, v)
                    except Exception as e:
                        print(type(attr).__name__)
                        raise e

    def _encode_data(self, header: header_pb2.Header, message: dict = None, pbMessage=None):
        if not header:
            return None
        messageData = None
        messageCrc = None
        if message and pbMessage:
            self._bind_message(pbMessage, message)
            messageData = pbMessage.SerializeToString()
            messageCrc = bytes([self._calculateCrc(messageData)])
            self._log("Message:\n%s" % pbMessage)
        if messageData:
            setattr(header, 'dataLen', len(messageData))
        else:
            setattr(header, 'dataLen', 0)
        self._log("Header:\n%s" % header)
        headerData = header.SerializeToString()
        headerCrc = bytes([self._calculateCrc(headerData)])
        frameHeader = bytes([0xDB, len(headerData)])
        if messageData:
            return frameHeader + headerData + headerCrc + messageData + messageCrc
        else:
            return frameHeader + headerData + headerCrc

    def _find_header_byte(self, data: bytes):
        for i in range(len(self._data)):
            if data[i] == 0xDB:
                return i
        return -1

    def _sub_data(self, data: bytes, offset: int, length: int = None):
        if length:
            return data[offset:offset + length]
        else:
            return data[offset:]

    def _decode_data(self, data: bytes):
        try:
            self._receive_lock.acquire()
            self._data = self._data + data
            if len(self._data) == 0:
                return None, None
            baseIndex = self._find_header_byte(self._data)
            if baseIndex < 0:
                # 找不到DB，继续等待数据，清空所有数据
                self._data = bytes([])
                return None, None
            length = len(self._data)
            if baseIndex + 1 >= length:
                # 数据不完整，继续等待数据
                return None, None
            # byteHeaderFlag = self._data[baseIndex + 0]
            byteHeaderLen = self._data[baseIndex + 1]
            if baseIndex + 3 + byteHeaderLen > length:
                # 数据不完整，继续等待数据
                return None, None
            byteHeaderData = self._sub_data(self._data, baseIndex + 2, byteHeaderLen)
            byteHeaderCrc = self._data[baseIndex + 2 + byteHeaderLen]
            calculateHeaderCrc = self._calculateCrc(byteHeaderData)
            if calculateHeaderCrc != byteHeaderCrc:
                # crc校验失败，从缓存数据中移除当前header
                self._data = self._sub_data(self._data, baseIndex + 3 + byteHeaderLen)
                return None, None
            header = header_pb2.Header()
            header.ParseFromString(byteHeaderData)
            payloadLen = getattr(header, 'dataLen')
            if payloadLen > 0:
                if baseIndex + 4 + byteHeaderLen + payloadLen > length:
                    # 数据不完整，继续等待数据
                    return None, None
                bytePayloadData = self._sub_data(self._data, baseIndex + 3 + byteHeaderLen, payloadLen)
                bytePayloadCrc = self._data[baseIndex + 3 + byteHeaderLen + payloadLen]
                calculatePayloadCrc = self._calculateCrc(bytePayloadData)
                # 数据接收完整，从缓存数据中移除当前header和payload
                self._data = self._sub_data(self._data, baseIndex + 4 + byteHeaderLen + payloadLen)
                if calculatePayloadCrc != bytePayloadCrc:
                    # crc校验失败
                    return None, None
                else:
                    return header, bytePayloadData
            else:
                # 数据接收完整，从缓存数据中移除当前header
                self._data = self._sub_data(self._data, baseIndex + 3 + byteHeaderLen)
                return header, None
        finally:
            self._receive_lock.release()

    def _get_seq(self):
        seq = self._seq
        self._seq = self._seq + 1
        return seq

    def _calculateCrc(self, data: bytes):
        return self.__calculateCrc(data, 0, len(data))

    def __calculateCrc(self, data: bytes, offset: int, length: int):
        crc = 0
        for i in range(length):
            crc ^= data[i + offset]
            for j in range(8):
                if crc & 0x80:
                    crc = crc << 1 ^ 0x07
                else:
                    crc = crc << 1
        return (crc ^ 0x55) & 0xFF

    def _sendRequest(self, dev: int, id: int, cmd: int, message: dict, pbReq, pbRsp, timeout_ms: int = 3000) -> tuple:
        try:
            self._send_lock.acquire()
            header = header_pb2.Header()
            seq = self._get_seq()
            setattr(header, 'dev', dev)
            setattr(header, 'id', id)
            setattr(header, 'cmd', cmd)
            setattr(header, 'attr', 1)  # hardcode attr=REQUEST
            setattr(header, 'ack', 0)  # hardcode ack=0
            setattr(header, 'seq', seq)
            try:
                data = self._encode_data(header, message, pbReq)
            except:
                return -2, None, None
            if not data:
                return -1, None, None
            self._dump(data, " >>>>> Send:")
            self._sendData(data)
            endtime_ms = (time.time() * 1000.0) + timeout_ms
            while True:
                remaining = endtime_ms - (time.time() * 1000.0)
                if remaining <= 0.0:
                    return -9, None, None
                if not self._queue.empty():
                    header, messageData = self._queue.get_nowait()
                    if getattr(header, 'attr') == 2 and getattr(header, 'seq') == seq:
                        ack = getattr(header, 'ack')
                        if ack == 0:
                            if pbRsp:
                                if messageData:
                                    pbRsp.ParseFromString(messageData)
                                else:
                                    pbRsp.ParseFromString(bytes([]))
                                response = pbRsp
                            else:
                                response = None
                            return ack, response, header
                        else:
                            return ack, None, header
                time.sleep(0.01)
        finally:
            self._send_lock.release()

    def sendRequest(self, dev: int, id: int, cmd: int, message: dict, pbReq, pbRsp, timeout_ms: int = 3000) -> tuple:
        ack, response, header = self._sendRequest(dev, id, cmd, message, pbReq, pbRsp, timeout_ms)
        if header is None:
            dataLen = 0
        else:
            dataLen = getattr(header, 'dataLen')
        self._log("Dev: %d, Cmd: %d, Id: %d, Result: %d, DataLen: %d" % (dev, cmd, id, ack, dataLen))
        if response:
            self._log("Response:\n%s" % response)
        return ack, response

    def _onReceivedData(self, data: bytes):
        self._dump(data, " <<<<< Receive:")
        header, messageData = self._decode_data(data)
        self._onReceivedResponse(header, messageData)

    def _onReceivedResponse(self, header: header_pb2.Header, messageData: bytes):
        if header:
            self._queue.put_nowait((header, messageData))



from enum import Enum

from ukitai.apis import *
from ukitai.common import *
from ukitai.link import *
from ukitai.protos import *


class Fruit(Enum):
    """水果

    APPLE: 苹果

    BANANA: 香蕉

    ORANGE: 橙子

    WATERMELON: 西瓜

    GRAPE: 葡萄

    KIWI: 猕猴桃

    PEAR: 梨

    PITAYA: 火龙果

    STRAWBERRY: 草莓

    PINEAPPLE: 菠萝
    """
    APPLE = 1
    BANANA = 2
    ORANGE = 3
    WATERMELON = 4
    GRAPE = 5
    KIWI = 6
    PEAR = 7
    PITAYA = 8
    STRAWBERRY = 9
    PINEAPPLE = 10


class TrafficSign(Enum):
    """交通标志

    RED: 红灯

    GREEN: 绿灯

    YELLOW: 黄灯

    TURN_LEFT: 左转

    TURN_RIGHT: 右转

    HORN: 鸣笛

    WATCH_OUT: 注意儿童

    NO_STOPPING: 禁止长时间停车

    TUNNEL: 进入隧道
    """
    RED = 1
    GREEN = 2
    YELLOW = 3
    TURN_LEFT = 4
    TURN_RIGHT = 5
    HORN = 6
    WATCH_OUT = 7
    NO_STOPPING = 8
    TUNNEL = 9


class Emotion(Enum):
    """表情图案

    SMILE: 微笑

    SAD: 难过

    BLINK: 眨眼

    SHY: 害羞

    SURPRISED: 惊讶

    DIZZY: 晕

    SIGH: 叹气

    LAUGH: 大笑
    """
    SMILE = 1
    SAD = 2
    BLINK = 3
    SHY = 4
    SURPRISED = 5
    DIZZY = 6
    SIGH = 7
    LAUGH = 8


class Color(Enum):
    """颜色定义

    WHITE: 白色

    PURPLE: 紫色

    RED: 红色

    ORANGE: 橙色

    YELLOW: 黄色

    GREEN: 绿色

    CYAN: 青色

    BLUE: 蓝色

    BLACK: 黑色
    """
    WHITE = 0x07
    PURPLE = 0x06
    RED = 0x00
    ORANGE = 0x01
    YELLOW = 0x02
    GREEN = 0x03
    CYAN = 0x05
    BLUE = 0x04
    BLACK = 0x08


class SensorType(Enum):
    """传感器类型

    INFRARED: 红外传感器

    ULTRASONIC: 超声传感器

    SOUND: 声音传感器

    LIGHT: 亮度传感器

    COLOR: 颜色传感器

    HUMITURE: 温湿度传感器
    """
    INFRARED = 0x5
    ULTRASONIC = 0x9
    SOUND = 0x7
    LIGHT = 0x11
    COLOR = 0xd
    HUMITURE = 0x6


def set_state(*, enabled: bool, link: uKitAiLink):
    """打开/关闭屏幕功能

    参数值说明:
        enabled: True=开打屏幕功能，False=关闭屏幕功能

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    return CommonApi.comp_state(compType=CompType.LCD, id=1, enabled=enabled, link=link)


def off(*, link: uKitAiLink):
    """控制屏幕关闭

    参数值说明:
        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    return set_state(enabled=False, link=link)


def clear(*, link: uKitAiLink):
    """控制屏幕清屏

    参数值说明:
        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = None
    pbReq = d1B_Lcd_pb2.d1bc1001_lcd_clear_rq()
    pbRsp = d1B_Lcd_pb2.d1bc1001_lcd_clear_ra()
    return link.sendRequest(CompType.LCD, 1, 1001, message, pbReq, pbRsp)


def set_brightness(*, brightness: int, link: uKitAiLink):
    """控制屏幕亮度

    参数值说明:
        brightness: 亮度值，范围[0, 100]

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'value': brightness}
    pbReq = d1B_Lcd_pb2.d1bc1002_lcd_back_light_set_rq()
    pbRsp = d1B_Lcd_pb2.d1bc1002_lcd_back_light_set_ra()
    return link.sendRequest(CompType.LCD, 1, 1002, message, pbReq, pbRsp)


def set_background_color(*, color: Color, link: uKitAiLink):
    """控制屏幕亮度

    参数值说明:
        color: 背景色，定义在 Screen.Color 中

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'color': color.value}
    pbReq = d1B_Lcd_pb2.d1bc1003_lcd_background_color_set_rq()
    pbRsp = d1B_Lcd_pb2.d1bc1003_lcd_background_color_set_ra()
    return link.sendRequest(CompType.LCD, 1, 1003, message, pbReq, pbRsp)


# def set_status_info(*, visible: bool, wifi_ssid: str = '', link: uKitAiLink):
def set_status_info(*, visible: bool, link: uKitAiLink):
    """控制屏幕状态界面显示

    参数值说明:
        visible: 状态界面是否可见

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'status': visible}
    pbReq = d1B_Lcd_pb2.d1bc1004_lcd_bar_display_rq()
    pbRsp = d1B_Lcd_pb2.d1bc1004_lcd_bar_display_ra()
    return link.sendRequest(CompType.LCD, 1, 1004, message, pbReq, pbRsp)


def show_fruit(*, fruit: Fruit, link: uKitAiLink):
    """控制屏幕显示水果

    参数值说明:
        fruit: 水果, 定义在 Screen.Fruit 中

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'index': fruit.value}
    pbReq = d1B_Lcd_pb2.d1bc1008_lcd_inner_pic_rq()
    pbRsp = d1B_Lcd_pb2.d1bc1008_lcd_inner_pic_ra()
    return link.sendRequest(CompType.LCD, 1, 1008, message, pbReq, pbRsp)


def show_traffic_sign(*, traffic_sign: TrafficSign, x: int = 0, y: int = 0, link: uKitAiLink):
    """控制屏幕显示交通标志

    参数值说明:
        traffic_sign: 交通标志, 定义在 Screen.TrafficSign 中

        x: 屏幕水平方向坐标

        y: 屏幕垂直方向坐标

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'type': 0, 'index': traffic_sign.value, 'post_x': x, 'post_y': y}
    pbReq = d1B_Lcd_pb2.d1bc1007_lcd_icon_rq()
    pbRsp = d1B_Lcd_pb2.d1bc1007_lcd_icon_rq()
    return link.sendRequest(CompType.LCD, 1, 1007, message, pbReq, pbRsp)


def show_emotion(*, emotion: Emotion, x: int = 0, y: int = 0, link: uKitAiLink):
    """控制屏幕显示表情

    参数值说明:
        emotion: 表情, 定义在 Screen.Emotion 中

        x: 屏幕水平方向坐标

        y: 屏幕垂直方向坐标

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'type': 1, 'index': emotion.value, 'post_x': x, 'post_y': y}
    pbReq = d1B_Lcd_pb2.d1bc1007_lcd_icon_rq()
    pbRsp = d1B_Lcd_pb2.d1bc1007_lcd_icon_rq()
    return link.sendRequest(CompType.LCD, 1, 1007, message, pbReq, pbRsp)


def show_words(*, text: str, color: Color = Color.WHITE, x: int = 0, y: int = 0, link: uKitAiLink):
    """控制屏幕显示静态文本文字

    参数值说明:
        text: 文字内容

        color: 文字颜色, 定义在 Screen.Color 中

        x: 屏幕水平方向坐标

        y: 屏幕垂直方向坐标

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'text': Common.str_to_gbk(text), 'color': color.value, 'post_x': x, 'post_y': y}
    pbReq = d1B_Lcd_pb2.d1bc1005_lcd_static_text_rq()
    pbRsp = d1B_Lcd_pb2.d1bc1005_lcd_static_text_ra()
    return link.sendRequest(CompType.LCD, 1, 1005, message, pbReq, pbRsp)


def show_marquee_words(*, text: str, color: Color = Color.WHITE, x: int = 0, y: int = 0, link: uKitAiLink):
    """控制屏幕显示动态滚动文本文字

    参数值说明:
        text: 文字内容

        color: 文字颜色, 定义在 Screen.Color 中

        x: 屏幕水平方向坐标

        y: 屏幕垂直方向坐标

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'text': Common.str_to_gbk(text), 'color': color.value, 'post_x': x, 'post_y': y}
    pbReq = d1B_Lcd_pb2.d1bc1006_lcd_roll_text_rq()
    pbRsp = d1B_Lcd_pb2.d1bc1006_lcd_roll_text_ra()
    return link.sendRequest(CompType.LCD, 1, 1006, message, pbReq, pbRsp)


def start_display_sensor_data(*, sensor_type: SensorType, link: uKitAiLink):
    """控制屏幕开始显示传感器数值

    参数值说明:
        sensor_type: 传感器类型，定义在 Screen.SensorType 中

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'opt': 1, 'dev': sensor_type.value}
    pbReq = d1B_Lcd_pb2.d1bc1010_lcd_sensor_display_rq()
    pbRsp = d1B_Lcd_pb2.d1bc1010_lcd_sensor_display_ra()
    return link.sendRequest(CompType.LCD, 1, 1010, message, pbReq, pbRsp)


def stop_display_sensor_data(*, link: uKitAiLink):
    """控制屏幕关闭显示传感器数值

    参数值说明:
        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'opt': 0}
    pbReq = d1B_Lcd_pb2.d1bc1010_lcd_sensor_display_rq()
    pbRsp = d1B_Lcd_pb2.d1bc1010_lcd_sensor_display_ra()
    return link.sendRequest(CompType.LCD, 1, 1010, message, pbReq, pbRsp)


def show_custom_emotion(*, name: str, link: uKitAiLink):
    """控制屏幕显示自定义表情

    参数值说明:
        name: 自定义表情名称

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'type': 0, 'name': name}
    pbReq = d1B_Lcd_pb2.d1bc1009_lcd_pic_display_rq()
    pbRsp = d1B_Lcd_pb2.d1bc1009_lcd_pic_display_ra()
    return link.sendRequest(CompType.LCD, 1, 1009, message, pbReq, pbRsp)


def show_custom_picture(*, name: str, link: uKitAiLink):
    """控制屏幕显示自定义图片

    参数值说明:
        name: 自定义图片名称

        link: 设备连接

    返回值说明:
        (ack, response): ack: 接口调用结果，response: 接口返回数据
    """
    message = {'type': 1, 'name': name}
    pbReq = d1B_Lcd_pb2.d1bc1009_lcd_pic_display_rq()
    pbRsp = d1B_Lcd_pb2.d1bc1009_lcd_pic_display_ra()
    return link.sendRequest(CompType.LCD, 1, 1009, message, pbReq, pbRsp)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dD_Color.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x64\x44_Color.proto\"\x18\n\x16\x64\x64\x63\x31\x30\x30\x30_clr_rgb_get_rq\"&\n\x16\x64\x64\x63\x31\x30\x30\x30_clr_rgb_get_ra\x12\x0c\n\x04rgbc\x18\x01 \x01(\rb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dD_Color_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DDC1000_CLR_RGB_GET_RQ._serialized_start=18
  _DDC1000_CLR_RGB_GET_RQ._serialized_end=42
  _DDC1000_CLR_RGB_GET_RA._serialized_start=44
  _DDC1000_CLR_RGB_GET_RA._serialized_end=82
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dF_Battery.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10\x64\x46_Battery.proto\"\x18\n\x16\x64\x66\x63\x31\x30\x30\x30_bat_pwr_pct_rq\"-\n\x16\x64\x66\x63\x31\x30\x30\x30_bat_pwr_pct_ra\x12\x13\n\x0bpwr_percent\x18\x01 \x01(\r\"\x1b\n\x19\x64\x66\x63\x31\x30\x30\x31_bat_status_get_rq\"G\n\x19\x64\x66\x63\x31\x30\x30\x31_bat_status_get_ra\x12\x12\n\nbat_status\x18\x01 \x01(\r\x12\x16\n\x0e\x61\x64\x61pter_status\x18\x02 \x01(\rb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dF_Battery_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DFC1000_BAT_PWR_PCT_RQ._serialized_start=20
  _DFC1000_BAT_PWR_PCT_RQ._serialized_end=44
  _DFC1000_BAT_PWR_PCT_RA._serialized_start=46
  _DFC1000_BAT_PWR_PCT_RA._serialized_end=91
  _DFC1001_BAT_STATUS_GET_RQ._serialized_start=93
  _DFC1001_BAT_STATUS_GET_RQ._serialized_end=120
  _DFC1001_BAT_STATUS_GET_RA._serialized_start=122
  _DFC1001_BAT_STATUS_GET_RA._serialized_end=193
# @@protoc_insertion_point(module_scope)




__all__ = [
    'd11_Luxmeter_pb2',
    'd1A_LedBelt_pb2',
    'd1B_Lcd_pb2',
    'd20_VisionModule_pb2',
    'd3_Servo_pb2',
    'd4_Motor_pb2',
    'd5_Infrared_pb2',
    'd6_TempHum_pb2',
    'd7_Sound_pb2',
    'd8_Led_pb2',
    'd9_Ultrasonic_pb2',
    'dB_TouchSwitch_pb2',
    'dD_Color_pb2',
    'dE_VoiceRecognition_pb2',
    'dF_Battery_pb2',
    'header_pb2',
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: d4_Motor.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x64\x34_Motor.proto\"7\n\x18\x64\x34\x63\x31\x30\x30\x30_mtr_speed_set_rq\x12\r\n\x05speed\x18\x01 \x01(\x11\x12\x0c\n\x04time\x18\x02 \x01(\r\"\x1a\n\x18\x64\x34\x63\x31\x30\x30\x30_mtr_speed_set_ra\"\x15\n\x13\x64\x34\x63\x31\x30\x30\x31_mtr_stop_rq\"\x15\n\x13\x64\x34\x63\x31\x30\x30\x31_mtr_stop_ra\"%\n\x16\x64\x34\x63\x31\x30\x30\x34_mtr_pwm_set_rq\x12\x0b\n\x03pwm\x18\x01 \x01(\x11\"\x18\n\x16\x64\x34\x63\x31\x30\x30\x34_mtr_pwm_set_rab\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'd4_Motor_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _D4C1000_MTR_SPEED_SET_RQ._serialized_start=18
  _D4C1000_MTR_SPEED_SET_RQ._serialized_end=73
  _D4C1000_MTR_SPEED_SET_RA._serialized_start=75
  _D4C1000_MTR_SPEED_SET_RA._serialized_end=101
  _D4C1001_MTR_STOP_RQ._serialized_start=103
  _D4C1001_MTR_STOP_RQ._serialized_end=124
  _D4C1001_MTR_STOP_RA._serialized_start=126
  _D4C1001_MTR_STOP_RA._serialized_end=147
  _D4C1004_MTR_PWM_SET_RQ._serialized_start=149
  _D4C1004_MTR_PWM_SET_RQ._serialized_end=186
  _D4C1004_MTR_PWM_SET_RA._serialized_start=188
  _D4C1004_MTR_PWM_SET_RA._serialized_end=212
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: d3_Servo.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x64\x33_Servo.proto\"j\n\x18\x64\x33\x63\x31\x30\x30\x30_srv_angle_set_rq\x12\x11\n\ttar_angle\x18\x01 \x01(\x11\x12\x15\n\rrotation_time\x18\x02 \x01(\r\x12\x16\n\x0eshielding_time\x18\x03 \x01(\r\x12\x0c\n\x04mode\x18\x04 \x01(\r\"\x1a\n\x18\x64\x33\x63\x31\x30\x30\x30_srv_angle_set_ra\"\'\n\x18\x64\x33\x63\x31\x30\x30\x31_srv_angle_get_rq\x12\x0b\n\x03pwr\x18\x01 \x01(\x08\"@\n\x18\x64\x33\x63\x31\x30\x30\x31_srv_angle_get_ra\x12\x11\n\tcur_angle\x18\x01 \x01(\x11\x12\x11\n\ttar_angle\x18\x02 \x01(\x11\"%\n\x16\x64\x33\x63\x31\x30\x30\x32_srv_pwm_set_rq\x12\x0b\n\x03pwm\x18\x01 \x01(\x11\"\x18\n\x16\x64\x33\x63\x31\x30\x30\x32_srv_pwm_set_ra\"#\n\x13\x64\x33\x63\x31\x30\x30\x34_srv_stop_rq\x12\x0c\n\x04mode\x18\x01 \x01(\r\"\x15\n\x13\x64\x33\x63\x31\x30\x30\x34_srv_stop_rab\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'd3_Servo_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _D3C1000_SRV_ANGLE_SET_RQ._serialized_start=18
  _D3C1000_SRV_ANGLE_SET_RQ._serialized_end=124
  _D3C1000_SRV_ANGLE_SET_RA._serialized_start=126
  _D3C1000_SRV_ANGLE_SET_RA._serialized_end=152
  _D3C1001_SRV_ANGLE_GET_RQ._serialized_start=154
  _D3C1001_SRV_ANGLE_GET_RQ._serialized_end=193
  _D3C1001_SRV_ANGLE_GET_RA._serialized_start=195
  _D3C1001_SRV_ANGLE_GET_RA._serialized_end=259
  _D3C1002_SRV_PWM_SET_RQ._serialized_start=261
  _D3C1002_SRV_PWM_SET_RQ._serialized_end=298
  _D3C1002_SRV_PWM_SET_RA._serialized_start=300
  _D3C1002_SRV_PWM_SET_RA._serialized_end=324
  _D3C1004_SRV_STOP_RQ._serialized_start=326
  _D3C1004_SRV_STOP_RQ._serialized_end=361
  _D3C1004_SRV_STOP_RA._serialized_start=363
  _D3C1004_SRV_STOP_RA._serialized_end=384
# @@protoc_insertion_point(module_scope)



from ukitai.common import *


def num_normal(value, upper_limit, lower_limit, mode=RangMode.NORMAL):
    """规范化数值

    参数值说明:
        value: 原始值

        upper_limit: 数值上限

        lower_limit: 数值下限

        mode (RangMode, optional): 规范模式. Defaults to RangMode.NORMAL.

    返回值说明:
        返回规范后的值. 如果 value 超出了 [lower_limit, upper_limit], 会被置为 lower_limit
        或 upper_limit, 然后根据 mode 进行改变

    示例:
        k = num_normal(5, 3, 4) # k = 5
        p = num_normal(5, 8, 10, RangMode.OPPOSITE) # p = -8
        n = num_normal(-5, -10, -6, RangMode.ABSOLUTE) # n = 6
    """
    # 确保 上限 不低于 下限
    if upper_limit < lower_limit:
        (upper_limit, lower_limit) = (lower_limit, upper_limit)

    if value > upper_limit:
        value = upper_limit
    if value < lower_limit:
        value = lower_limit
    if mode == RangMode.ABSOLUTE:
        value = abs(value)
    elif mode == RangMode.OPPOSITE:
        value = value * (-1)

    return value


def str_to_gbk(s: str):
    """将字符串转码为GBK编码"""
    return s.encode('gbk')



INFRARED_DISTANCE = 0
ULTRASONIC_DISTANCE = 1
SOUND_STRONG = 2
BRIGHTNESS = 3
HUMIDITY = 4
TEMPERATURE_F = 6
TEMPERATURE_C = 5
COLOR_RED = 7
COLOR_GREEN = 8
COLOR_BLUE = 9
SERVO_ANGLE = 10
BATTERY_LEVEL = 11
BATTERY_STATUS = 12
CHARGE_STATUS = 13
TOUCH_STATUS = 14
